import json
import re
from collections import namedtuple

import numpy as np
import pyopenms
from ErrorWidget import ErrorWidget
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QHBoxLayout, QWidget, QSplitter
from ScanTableWidget import ScanTableWidget
from SequenceIonsWidget import SequenceIonsWidget
from SpectrumWidget import SpectrumWidget
from TICWidget import TICWidget

PeakAnnoStruct = namedtuple(
    "PeakAnnoStruct",
    "mz intensity text_label \
                            symbol symbol_color",
)
LadderAnnoStruct = namedtuple(
    "LadderAnnoStruct",
    "mz_list \
                            text_label_list color",
)


class ControllerWidget(QWidget):
    """
    Used to merge spectrum, table, TIC,
    error plot and sequenceIons widgets together.

    """

    def __init__(self, *args, **kwargs):
        QWidget.__init__(self, *args, **kwargs)
        self.mainlayout = QHBoxLayout(self)
        self.isAnnoOn = True
        self.clickedRT = None
        self.seleTableRT = None
        self.mzs = np.array([])
        self.ppm = np.array([])
        self.colors = np.array([])
        self.scanIDDict = {}
        self.curr_table_index = None
        self.filteredIonFragments = []
        self.peakAnnoData = None

    def clearLayout(self, layout):
        for i in reversed(range(layout.count())):
            layout.itemAt(i).widget().setParent(None)

    def loadFileMzML(self, file_path):
        self.isAnnoOn = False
        self.msexperimentWidget = QSplitter(Qt.Vertical)

        # data processing
        scans = self.readMS(file_path)

        # set Widgets
        self.spectrum_widget = SpectrumWidget()
        self.scan_widget = ScanTableWidget(scans)
        self.seqIons_widget = SequenceIonsWidget()
        self.error_widget = ErrorWidget()
        self.tic_widget = TICWidget()
        self.drawTic(scans)

        # connected signals
        self.scan_widget.sigScanClicked.connect(self.updateWidgetDataFromRow)
        self.tic_widget.sigRTClicked.connect(self.ticToTable)

        self.msexperimentWidget.addWidget(self.tic_widget)
        self.msexperimentWidget.addWidget(self.seqIons_widget)
        self.msexperimentWidget.addWidget(self.spectrum_widget)
        self.msexperimentWidget.addWidget(self.error_widget)
        self.msexperimentWidget.addWidget(self.scan_widget)
        self.mainlayout.addWidget(self.msexperimentWidget)

        # set widget sizes, where error plot is set smaller
        widget_height = self.msexperimentWidget.sizeHint().height()
        size_list = [
            widget_height,
            widget_height,
            widget_height,
            widget_height * 0.5,
            widget_height
        ]
        self.msexperimentWidget.setSizes(size_list)

        # default : first row selected.
        self.scan_widget.table_view.selectRow(0)

    def loadFileIdXML(self, file_path):
        prot_ids = []
        pep_ids = []
        pyopenms.IdXMLFile().load(file_path, prot_ids, pep_ids)
        Ions = {}

        # extract ID data from file
        for peptide_id in pep_ids:
            pep_mz = peptide_id.getMZ()
            pep_rt = peptide_id.getRT()

            for hit in peptide_id.getHits():
                pep_seq = str(hit.getSequence().toString())
                if "." in pep_seq:
                    pep_seq = pep_seq[3:-1]
                else:
                    pep_seq = pep_seq[2:-1]

                for anno in hit.getPeakAnnotations():
                    ion_charge = anno.charge
                    ion_mz = anno.mz
                    ion_label = anno.annotation

                    Ions[ion_label] = [ion_mz, ion_charge]

                self.scanIDDict[round(pep_rt, 3)] = {
                    "m/z": pep_mz,
                    "PepSeq": pep_seq,
                    "PepIons": Ions,
                }
                Ions = {}

        self.saveIdData()

    def saveIdData(self):
        # save ID data in table (correct rows) for later usage
        rows = self.scan_widget.table_model.rowCount(self.scan_widget)

        for row in range(0, rows - 1):
            tableRT = round(
                self.scan_widget.table_model.index(row, 2).data(), 3)
            if tableRT in self.scanIDDict:
                index_seq = self.scan_widget.table_model.index(row, 6)
                self.scan_widget.table_model.setData(
                    index_seq, self.scanIDDict[tableRT]["PepSeq"],
                    Qt.DisplayRole
                )

                index_ions = self.scan_widget.table_model.index(row, 7)
                # data needs to be a string, but reversible ->
                # using json.dumps()
                self.scan_widget.table_model.setData(
                    index_ions,
                    json.dumps(self.scanIDDict[tableRT]["PepIons"]),
                    Qt.DisplayRole,
                )

    def readMS(self, file_path):
        # read MzML files
        exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(file_path, exp)
        return exp

    def drawTic(self, scans):
        self.tic_widget.setTIC(scans.getTIC())

    def ticToTable(self, rt):
        # connect Tic info to table, and select specific row
        self.clickedRT = round(rt * 60, 3)
        if self.clickedRT != self.seleTableRT:
            self.scan_widget.table_view.selectRow(self.findClickedRT())

    def findClickedRT(self):  # find clicked RT in the scan table
        rows = self.scan_widget.table_model.rowCount(self.scan_widget)

        for row in range(0, rows - 1):
            if self.clickedRT == round(
                    self.scan_widget.table_model.index(row, 2).data(), 3
            ):
                index = self.scan_widget.table_model.index(row, 2)
                try:
                    self.curr_table_index \
                        = self.scan_widget.proxy.mapFromSource(index)
                    # use proxy to get from filtered model index
                    return self.curr_table_index.row()
                except ValueError:
                    print("could not found ModelIndex of row")

    # for the future calculate ppm and add it to the table
    def errorData(self, ions_data):
        if ions_data not in "-":
            ions_data_dict = json.loads(ions_data)
            if ions_data_dict != {}:
                self.colors, self.mzs = self.filterColorsMZIons(ions_data_dict)
                mzs_size = len(self.mzs)
                self.ppm = np.random.randint(0, 3, size=mzs_size)
                self.error_widget.setMassErrors(
                    self.mzs, self.ppm, self.colors
                )  # works for a static np.array
            else:
                self.error_widget.clear()
        else:
            self.error_widget.clear()

    def filterColorsMZIons(
            self, ions_data_dict
    ):  # create color/mz array by distinguishing between prefix & suffix ions
        self.peakAnnoData = (
            {}
        )  # key is ion annotation (e.g. b2):
        # [mz, color distinguishing prefix, suffix]
        colors = []
        mzs = []
        col_red = (255, 0, 0)  # suffix
        col_blue = (0, 0, 255)  # prefix

        for fragData in self.filteredIonFragments:
            anno = fragData[0]
            if anno[0] in "abc":
                colors.append(col_blue)
                mzs.append(ions_data_dict[anno][0])
                self.peakAnnoData[fragData[1]] = [
                    ions_data_dict[anno][0], col_blue]
            elif anno[0] in "xyz":
                colors.append(col_red)
                mzs.append(ions_data_dict[anno][0])
                self.peakAnnoData[fragData[1]] = [
                    ions_data_dict[anno][0], col_red]
        return np.array(colors), np.array(mzs)

    def updateWidgetDataFromRow(
            self, index
    ):  # after clicking on a new row, update spectrum, error plot, peptideSeq
        # current row RT value
        self.seleTableRT = round(index.siblingAtColumn(2).data(), 3)

        # set new spectrum with setting that all peaks should be displayed
        self.spectrum_widget.setSpectrum(
            self.scan_widget.curr_spec, zoomToFullRange=True
        )

        # only draw sequence with given ions for MS2 and error plot
        if index.siblingAtColumn(0).data() == "MS2":
            self.drawSeqIons(
                index.siblingAtColumn(
                    6).data(), index.siblingAtColumn(7).data()
            )
            self.errorData(index.siblingAtColumn(7).data())
            if (
                    self.peakAnnoData is not None
            ):  # peakAnnoData created with existing ions in errorData
                # (bc of coloring)
                self.spectrum_widget.setPeakAnnotations(
                    self.createPeakAnnotation())
                self.spectrum_widget.redrawPlot()
            else:
                self.spectrum_widget._clear_peak_annotations()
                self.spectrum_widget.redrawPlot()

        # otherwise delete old data
        elif index.siblingAtColumn(0).data() == "MS1":
            self.seqIons_widget.clear()
            self.error_widget.clear()
            self.peakAnnoData = None
            self.spectrum_widget._clear_peak_annotations()
            self.spectrum_widget.redrawPlot()

    def createPeakAnnotation(self):
        pStructList = []
        # for the future ->
        # check clashes like in the TIC widget and then add labels
        # (should be done in SpectrumWidget)
        for anno, data in self.peakAnnoData.items():
            mz, anno_color = data[0], data[1]
            index = self.find_nearest_Index(self.spectrum_widget._mzs, mz)
            pStructList.append(
                PeakAnnoStruct(
                    mz=self.spectrum_widget._mzs[index],
                    intensity=self.spectrum_widget._ints[index],
                    text_label=anno,
                    symbol=None,
                    symbol_color=anno_color,
                )
            )
        return pStructList

    def find_nearest_Index(self, array, value):
        array = np.asarray(array)
        idx = (np.abs(array - value)).argmin()
        return idx

    def drawSeqIons(self, seq, ions):  # generate provided peptide sequence
        seq = re.sub(
            r"\([^)]*\)", "", seq
        )  # remove content in brackets -> easier usage

        # only draw sequence for M2 with peptide and ion data
        if seq not in "-" and ions not in "-":
            self.seqIons_widget.setPeptide(seq)
            # transform string data back to a dict
            ions_dict = json.loads(ions)
            if ions_dict != {}:
                self.suffix, self.prefix = self.filterIonsPrefixSuffixData(
                    ions_dict)
                self.seqIons_widget.setPrefix(self.prefix)
                self.seqIons_widget.setSuffix(self.suffix)
            else:  # no ions data
                self.prefix, self.suffix = {}, {}
                self.seqIons_widget.setPrefix(self.prefix)
                self.seqIons_widget.setSuffix(self.suffix)
                self.peakAnnoData = None
        else:
            self.seqIons_widget.clear()
            self.peakAnnoData = None

    def filterIonsPrefixSuffixData(
            self, ions
    ):  # filter raw ion data and return suffix and prefix dicts
        suffix = {}
        prefix = {}

        ions_anno = list(ions.keys())
        # annotation(s) of raw ion data (used as key(s))
        self.filteredIonFragments = []

        for anno in ions_anno:
            if anno[1].isdigit() and anno[0] in "abcyxz":
                index, anno_short = self.filterAnnotationIon(anno)
                if (
                        (index in suffix) and
                        (anno[0] in "yxz") and
                        (anno_short not in suffix[index])
                ):  # avoid double annos e.g. y14
                    suffix[index].append(anno_short)
                elif (
                        (index in prefix) and
                        (anno[0] in "abc") and
                        (anno_short not in prefix[index])
                ):
                    prefix[index].append(anno_short)
                elif anno[0] in "yxz":  # non existing keys
                    suffix[index] = [anno_short]
                elif anno[0] in "abc":  # non existing keys
                    prefix[index] = [anno_short]
        return suffix, prefix

    def filterAnnotationIon(self, fragment_anno):
        # filter from raw ion data annotation index
        # and filtered annotation name (e.g. y2)
        index = [s for s in re.findall(r"-?\d+\.?\d*", fragment_anno)][0]
        ion_anno = fragment_anno.split(index)[0] + index
        self.filteredIonFragments.append((fragment_anno, ion_anno))
        return int(index), ion_anno



import sys
import os
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import (QWidget, QToolTip,
                             QPushButton, QApplication, QMainWindow,
                             QAction, qApp,
                             QHBoxLayout, QVBoxLayout, QMessageBox,
                             QLineEdit, QTableWidget, QTableWidgetItem,
                             QGridLayout, QPlainTextEdit,
                             QDesktopWidget, QLabel, QRadioButton,
                             QGroupBox, QSizePolicy, QCheckBox, QFileDialog,
                             QTextEdit, QTextBrowser)
from PyQt5.QtGui import QFont, QColor
#making a files dictionary that is going to be a global variable
files_dictionary = {

    "fasta": "",
    "tsv": "",
    "data": "",
    "mzML": "",
    "idXML": "",
    "ini_path" : ""
    }

booleans_dictionary = {

        "fasta": False,
        "tsv": False,
        "data": False,
        "mzML": False,
        "idXML": False,
        "ini_path" : False
        }

class Files_Number_Handler():
    #gets a folder path as an argument and searches for files that end with
    #.fasta or .tsv and saves them in the corresponding Array
    def Identify_Files_Numbers(folder_path):
        fasta_files=[]
        tsv_files = []
        mzML_files = []
        idXML_files= []
        ini_files = []

        fileslist = sorted(os.listdir(folder_path))
        #print(sorted(fileslist))
        for file in fileslist:
            if file.endswith(".fasta"):
                fasta_files.append(file)

            if file.endswith(".tsv"):
                tsv_files.append(file)

            if file.endswith(".mzML"):
                mzML_files.append(file)

            if file.endswith(".idXML"):
                idXML_files.append(file)

            if file.endswith(".ini"):
                ini_files.append(file)

        if len(mzML_files) == 0 and len(idXML_files) == 0:
            User_Warning = QMessageBox()
            User_Warning.setIcon(QMessageBox.Information)
            User_Warning.setText("No mzML and idXML files found. Please select a different folder.")
            User_Warning.setWindowTitle("Information")
            Information = User_Warning.exec_()


        if  len(mzML_files) == 0 and len(idXML_files) != 0:
            User_Warning = QMessageBox()
            User_Warning.setIcon(QMessageBox.Information)
            User_Warning.setText("No mzML files found. Please select a different folder.")
            User_Warning.setWindowTitle("Information")
            Information = User_Warning.exec_()
            Files_Number_Handler.Dictionary_Change_Boolean('idXML')

        if len(mzML_files) != 0 and len(idXML_files) == 0:
            User_Warning = QMessageBox()
            User_Warning.setIcon(QMessageBox.Information)
            User_Warning.setText("No idXML files found. Please select a different folder.")
            User_Warning.setWindowTitle("Information")
            Information = User_Warning.exec_()
            Files_Number_Handler.Dictionary_Change_Boolean('mzML')

        if len(mzML_files) != 0 and len(idXML_files) != 0:
            Files_Number_Handler.Dictionary_Change_Boolean('mzML')
            Files_Number_Handler.Dictionary_Change_Boolean('idXML')
            



        return fasta_files,tsv_files,mzML_files,idXML_files,ini_files

    #works just Identify_Files_Numbers but for the manualy option
    def Identify_Files_Numbers_Manualy(folder_path):

            mzML_files = []
            idXML_files= []
            mzMLLoaded = 0
            idXMLLoaded = 0
            fileslist = sorted(os.listdir(folder_path))
            for file in fileslist:
                if file.endswith(".mzML"):
                    mzML_files.append(file)

                if file.endswith(".idXML"):
                    idXML_files.append(file)


            if len(mzML_files) == 0 and len(idXML_files) == 0:
                User_Warning = QMessageBox()
                User_Warning.setIcon(QMessageBox.Information)
                User_Warning.setText("No mzML and idXML files found. Pleas select a different folder.")
                User_Warning.setWindowTitle("Information")
                Information = User_Warning.exec_()



            if  len(mzML_files) == 0 and len(idXML_files) != 0:
                User_Warning = QMessageBox()
                User_Warning.setIcon(QMessageBox.Information)
                User_Warning.setText("No mzML files found. Pleas select a different folder.")
                User_Warning.setWindowTitle("Information")
                Information = User_Warning.exec_()
                Files_Number_Handler.Dictionary_Change_Boolean('idXML')


            if len(mzML_files) != 0 and len(idXML_files) == 0:
                User_Warning = QMessageBox()
                User_Warning.setIcon(QMessageBox.Information)
                User_Warning.setText("No idXML files found. Pleas select a different folder.")
                User_Warning.setWindowTitle("Information")
                Information = User_Warning.exec_()
                Files_Number_Handler.Dictionary_Change_Boolean('mzML')

            if len(mzML_files) != 0 and len(idXML_files) != 0:
                Files_Number_Handler.Dictionary_Change_Boolean('mzML')
                Files_Number_Handler.Dictionary_Change_Boolean('idXML')



            return idXML_files,mzML_files

    #checks if array contains only on element, it is important because if
    #more than 1 user needs to select the on file he wants to use
    def Check_If_More_Than_One(arraytotest):
        if len(arraytotest)>1:
            return True
        else :
            return False

    def Check_If_Less_Than_One(arraytotest):
        if len(arraytotest)==0:
            return True
        else :
            return False
    def Check_If_One(arraytotest) :
        if len(arraytotest) == 1:
            return True
        else:
            return False


        #used to save paths of files when loaded manually from a tab widget
    def Dictionary_Change_File(file_type,file_path):
        global files_dictionary
        files_dictionary[file_type] = file_path
        #used to return the values of the dictionary
    def Dictionary_Return_Value(file_type):
        global files_dictionary
        return files_dictionary[file_type]
        #changes the value to true if , should be used whenever loading data
        #in to the dictionary manually
    def Dictionary_Change_Boolean(file_type):
        global booleans_dictionary
        booleans_dictionary[file_type] = True
        #returns the boolean
    def Dictionary_Return_Boolean(file_type):
        global booleans_dictionary
        return booleans_dictionary[file_type]



import sys

import pyopenms
from GUI_EXAMPLE_BASE import GUI_EXAMPLE_BASE
from PyQt5.QtWidgets import QApplication

sys.path.insert(0, "../view")
from ScanTableWidget import ScanTableWidget

if __name__ == "__main__":
    app = QApplication(sys.argv)
    ex = GUI_EXAMPLE_BASE()  # plain QMainWindow with basic layout and menu bar

    # load spectra and add example widget to window
    exp = pyopenms.MSExperiment()
    pyopenms.MzMLFile().load("../data/190509_Ova_native_25ngul_R.mzML", exp)
    example_widget = ScanTableWidget(exp)
    ex.setExampleWidget(example_widget)
    ex.show()
    sys.exit(app.exec_())



import numpy as np
import pyqtgraph as pg
from PyQt5.QtCore import pyqtSignal
from PyQt5.QtGui import QKeySequence
from PyQt5.QtWidgets import QShortcut
from pyqtgraph import PlotWidget

pg.setConfigOption("background", "w")  # white background
pg.setConfigOption("foreground", "k")  # black peaks


class TICWidget(PlotWidget):
    """
    Used for creating a TIC plot
    with dynamic zooming to avoid label collisions.

    ===============================  =========================================
    **Signals:**
    sigRTClicked                     Emitted when the user has clicked on TIC
                                     plot and returns the clicked RT value.

    sigSeleRTRegionChangeFinished    Emitted while the user is double clicking
                                     on a region in TIC plot and creates a
                                     region by dragging a horizontal line.
                                     The signal returns the start and end
                                     RT values within the region.
    ===============================  =========================================
    """

    sigRTClicked = pyqtSignal(float, name="sigRTClicked")
    sigSeleRTRegionChangeFinished = pyqtSignal(
        float, float, name="sigRTRegionChangeFinished"
    )

    def __init__(self, parent=None, dpi=100):
        PlotWidget.__init__(self)
        self.setLimits(yMin=0, xMin=0)
        self.setMouseEnabled(y=False)
        self.setLabel("bottom", "RT (min)")
        self.setLabel("left", "relative intensity (%)")
        self._peak_labels = {}
        self._existTIC = True
        # numpy arrays for fast look-up
        self._rts = np.array([])
        self._ints = np.array([])
        self._peak_indices = np.array([])
        self._currentIntensitiesInRange = np.array([])
        self._region = None
        self.getViewBox().sigXRangeChanged.connect(self._autoscaleYAxis)

        self.scene().sigMouseClicked.connect(self._clicked)  # emits rt_clicked

        # shortcut to init region
        self.shortcut1 = QShortcut(QKeySequence("Ctrl+r"), self)
        self.shortcut1.activated.connect(self._rgn_shortcut)

    # in cases only MS2 spectra are given
    def checkExistTIC(self):
        if self._rts.size == 0:
            self._existTIC = False

    def setTIC(self, chromatogram):
        """
        Used to set new TIC and with given Information (rts, ints)

        :param chromatogram: data from the MSExperiment
        """
        if self._peak_labels != {}:
            self._clear_labels()
            self._peak_labels = {}
        self._chrom = chromatogram
        self._rts, self._ints = self._chrom.get_peaks()

        self.checkExistTIC()
        if self._existTIC:
            self._rts_in_min()
            self._relative_ints()
            self._peak_indices = self._find_Peak()
            self._autoscaleYAxis()
            self.redrawPlot()

    def _rts_in_min(self):
        self._rts = np.array([x / 60 for x in self._rts])

    def _relative_ints(self):
        maxInt = np.amax(self._ints)
        self._ints = np.array([((x / maxInt) * 100) for x in self._ints])

    def redrawPlot(self):
        self.plot(clear=True)
        self._plot_tic()
        self._draw_peak_label()

    def _autoscaleYAxis(self):
        """
        Used to adjust y axis with the maximal y value
        from the current RT values. Also, redraws peak labels
        depending on the current displayed RT values.

        """
        x_range = self.getAxis("bottom").range
        if x_range == [0, 1]:  # workaround for axis sometimes not being set
            x_range = [np.amin(self._rts), np.amax(self._rts)]
        self.currMaxY = self._getMaxIntensityInRange(x_range)
        if self.currMaxY:
            self.setYRange(0, self.currMaxY, update=False)
            self._redrawLabels()

    def _getMaxIntensityInRange(self, xrange):
        """
        :param xrange: A list of [min, max] bounding RT values.
        :return: An float value representing the maximal
            intensity current x range.
        """
        left = np.searchsorted(self._rts, xrange[0], side="left")
        right = np.searchsorted(self._rts, xrange[1], side="right")
        self._currentIntensitiesInRange = self._ints[left:right]

        return np.amax(self._ints[left:right], initial=1)

    def _plot_tic(self):
        plotgraph = pg.PlotDataItem(self._rts, self._ints)
        self.addItem(plotgraph)

    def _find_Peak(self):
        """
        Calculates all indices from the intensity values to locate peaks.
        This function operates on the principle that it compares peak values
        against each other until it founds a maximal turning point.

        :return: A numpy array containing all peak indices,
            sorted descending (max first -> min last).
        """
        data = self._ints
        maxIndices = np.zeros_like(data)
        peakValue = -np.inf
        for indx in range(0, len(data), 1):
            if peakValue < data[indx]:
                peakValue = data[indx]
                for j in range(indx, len(data)):
                    if peakValue < data[j]:
                        break
                    elif peakValue == data[j]:
                        continue
                    elif peakValue > data[j]:
                        peakIndex = indx + np.floor(abs(indx - j) / 2)
                        # marking found index
                        maxIndices[peakIndex.astype(int)] = 1
                        indx = j
                        break
            peakValue = data[indx]
        maxIndices = np.where(maxIndices)[0]

        # sort indices of high points from largest intensity to smallest
        maxIndices = sorted(maxIndices, key=lambda x: data[x], reverse=True)

        return maxIndices

    def _add_label(self, label_id, label_text, pos_x, pos_y):
        label = pg.TextItem(anchor=(0.5, 1))
        label.setText(text="{0:.2f}".format(label_text), color=(0, 0, 0))
        label.setPos(pos_x, pos_y)
        self._peak_labels[label_id] = {"label": label}
        self.addItem(label, ignoreBounds=True)

        if self._label_clashes(label_id):
            self._remove_label(label_id)

    def _remove_label(self, label_id):
        self.removeItem(self._peak_labels[label_id]["label"])
        del self._peak_labels[label_id]

    def _clear_labels(self):
        for label_id in self._peak_labels.keys():
            self.removeItem(self._peak_labels[label_id]["label"])
        self._peak_labels = {}

    def _label_clashes(self, label_id):
        """
        Calculates possible clash of new added label to other existing labels.
        The clash is measured by the
        collision of the label boundingRects,
        which are representing displayed scene positions.

        :param label_id: Represents index of peak position in peak_indices.
        :return: A boolean indicating if there is a clash or not.
        """
        new_label = label_id
        clash = False

        # scaling the distance with the correct pixel size
        pixel_width = self.getViewBox().viewPixelSize()[0]
        limit_distance = 20.0 * pixel_width

        if self._peak_labels == {}:
            return False

        for exist_label in list(self._peak_labels):
            if exist_label != new_label:
                new_label_rect =\
                    self._peak_labels[new_label]["label"].mapRectToDevice(
                        self._peak_labels[new_label]["label"].boundingRect()
                    )
                exist_label_rect = self._peak_labels[exist_label][
                    "label"
                ].mapRectToDevice(
                    self._peak_labels[exist_label]["label"].boundingRect()
                )

                if not new_label_rect.intersects(exist_label_rect):
                    exist_label_X = self._peak_labels[exist_label]["label"].x()
                    new_label_X = self._peak_labels[new_label]["label"].x()

                    distance = abs(new_label_X - exist_label_X)

                    if distance < limit_distance:
                        clash = True
                        break
                    else:
                        clash = False

                elif new_label_rect.intersects(exist_label_rect):
                    clash = True
                    break
            else:
                if len(self._peak_labels) == 1 and exist_label == new_label:
                    clash = False
        return clash

    def _draw_peak_label(self):
        """
        Function draws peak labels,
        starting with the maximal peak to the minimal peak.
        In each addition possible label clashes will be calculated,
        if so then delete label.
        """
        if self._peak_labels == {}:
            for index in self._peak_indices:
                if self._ints[index] in self._currentIntensitiesInRange:
                    self._add_label(
                        index, self._rts[index],
                        self._rts[index],
                        self._ints[index]
                    )

    def _redrawLabels(self):
        self._clear_labels()
        self._draw_peak_label()

    def _clicked(self, event):
        if self._existTIC:
            pos = event.scenePos()
            if self.sceneBoundingRect().contains(pos):
                mouse_point = self.getViewBox().mapSceneToView(pos)
                closest_datapoint_idx = self._calculate_closest_datapoint(
                    mouse_point.x()
                )
                self.sigRTClicked.emit(
                    self._rts[closest_datapoint_idx]
                )  # notify observers

            # check the selected rt region and return the bounds
            if self._region is not None:
                self._region.sigRegionChangeFinished.connect(
                    self._rtRegionBounds)

    def mouseDoubleClickEvent(self, event):
        super(TICWidget, self).mouseDoubleClickEvent(event)
        try:
            mouse_point = self.getViewBox().mapSceneToView(event.pos())
            closest_datapoint_idx = self._calculate_closest_datapoint(
                mouse_point.x())
            rgn_start = self._rts[closest_datapoint_idx]

            if self._region is None:
                region = pg.LinearRegionItem()
                region.setRegion((rgn_start, rgn_start))
                self._region = region
                self.addItem(region, ignoreBounds=True)

            # delete the region when hovering over the region per doubleClk
            self._delete_region()
        except ValueError:
            print("No TIC values to click on")

    def _calculate_closest_datapoint(self, point_x):
        """
        :param point_x: mouse clicked position
        :return: closest data point near a peak

        """
        larger_idx = np.searchsorted(self._rts, point_x, side="left")
        smaller_idx = 0
        if larger_idx >= self._rts.size:  # to avoid array out of bounds
            larger_idx -= 1
        if larger_idx > 0:
            smaller_idx = larger_idx - 1
        if abs(self._rts[larger_idx] - point_x) < \
                abs(self._rts[smaller_idx] - point_x):
            closest_datapoint_idx = larger_idx

        else:
            closest_datapoint_idx = smaller_idx

        return closest_datapoint_idx

    def _rtRegionBounds(self):
        region_bounds = self._region.getRegion()
        start_rg = region_bounds[0]

        stop_rg_idx = self._calculate_closest_datapoint(region_bounds[1])
        stop_rg = self._rts[stop_rg_idx]

        # set the new region of interest
        self._region.setRegion((start_rg, stop_rg))

        self.sigSeleRTRegionChangeFinished.emit(
            start_rg, stop_rg)  # notify observers

    def _delete_region(self):
        if self._region.mouseHovering:
            self.removeItem(self._region)
            self._region = None

    def _rgn_shortcut(self):
        # click region, with following shortcut -> create region
        rgn_start = self.getViewBox().mapSceneToView(self.lastMousePos)

        if self._region is None:
            region = pg.LinearRegionItem()
            region.setRegion((rgn_start, rgn_start))
            self._region = region
            self.addItem(region, ignoreBounds=True)



import sys

import numpy as np
from GUI_EXAMPLE_BASE import GUI_EXAMPLE_BASE
from PyQt5.QtWidgets import QApplication

sys.path.insert(0, "../view")
from ErrorWidget import ErrorWidget

if __name__ == "__main__":
    app = QApplication(sys.argv)
    ex = GUI_EXAMPLE_BASE()  # plain QMainWindow with basic layout and menu bar

    # add example widget to window
    example_widget = ErrorWidget(ex)
    x = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
    y = np.array([-2, 3, -8, -4, 3, 12, -15, 10, 5, -12])
    green = (0, 255, 0)
    purple = (128, 0, 255)
    c = np.array(
        [green, purple, green, purple, green,
         purple, green, purple, green, green]
    )
    example_widget.setMassErrors(x, y, c)
    ex.setExampleWidget(example_widget)
    ex.show()
    sys.exit(app.exec_())



import numpy as np
import pyopenms
import pyqtgraph as pg
from pyqtgraph import PlotWidget


class MS1MapWidget(PlotWidget):
    def __init__(self, parent=None, dpi=100):
        PlotWidget.__init__(self)
        self.setLabel("bottom", "m/z")
        self.setLabel("left", "RT")

    def setSpectra(self, msexperiment):
        msexperiment.updateRanges()

        # resolution: mz_res Da in m/z, rt_res seconds in RT dimension
        mz_res = 1.0
        rt_res = 1.0

        # size of image
        cols = 1.0 / mz_res * msexperiment.getMaxMZ()
        rows = 1.0 / rt_res * msexperiment.getMaxRT()

        # create regular spaced data to turn spectra into an image
        """se_comment: max_intensity was never used"""
        """max_intensity = msexperiment.getMaxInt()"""
        bilip = pyopenms.BilinearInterpolation()
        tmp = bilip.getData()
        tmp.resize(int(rows), int(cols), float())
        bilip.setData(tmp)

        bilip.setMapping_0(0.0, 0.0, rows - 1, msexperiment.getMaxRT())
        bilip.setMapping_1(0.0, 0.0, cols - 1, msexperiment.getMaxMZ())

        img = pg.ImageItem(autoDownsample=True)
        self.addItem(img)

        for spec in msexperiment:
            if spec.getMSLevel() == 1:
                mzs, ints = spec.get_peaks()
                rt = spec.getRT()
                for i in range(0, len(mzs)):
                    bilip.addValue(rt, mzs[i], ints[i])  # slow

        data = np.ndarray(shape=(int(cols), int(rows)), dtype=np.float64)
        grid_data = bilip.getData()
        for i in range(int(rows)):
            for j in range(int(cols)):
                data[j][i] = grid_data.getValue(i, j)  # slow

        # Set a custom color map
        pos = np.array([0.0, 0.01, 0.05, 0.1, 1.0])
        color = np.array(
            [
                (255, 255, 255, 0),
                (255, 255, 0, 255),
                (255, 0, 0, 255),
                (0, 0, 255, 255),
                (0, 0, 0, 255),
            ],
            dtype=np.ubyte,
        )
        cmap = pg.ColorMap(pos, color)
        img.setLookupTable(cmap.getLookupTable(0.0, 1.0, 256))
        img.setImage(data)



  
import os
import sys
import timeit
import pandas as pd
import math
from PyQt5 import Qt

from PyQt5.QtWidgets import QHBoxLayout, QWidget, QFileDialog, \
        QTableWidget, QTableWidgetItem, QHeaderView, QPushButton, \
        QVBoxLayout, QInputDialog, QLineEdit, QMessageBox, \
        QAbstractItemView
sys.path.append(os.getcwd() + '/../controller')
from filehandler import FileHandler as fh  # noqa E402
sys.path.append(os.getcwd() + '/../model')
from tableDataFrame import TableDataFrame as Tdf  # noqa E402


class mzMLTableView(QWidget):
    """
    Main Widget of the TableEditor app
    """

    def __init__(self, *args):
        self.path = ""
        # set variable self.testForTime to True to see Runtimes
        # the following 2 if constructs can be used to determine
        # timing
        # just put them around whatever should be timed

        # self.testForTime = False
        # if self.testForTime:
        #    starttime = timeit.default_timer()
        #    print("Starttime of overall Initiation : ", starttime)

        # if self.testForTime:
        #    rt = timeit.default_timer() - starttime
        #    print("Runtime of overall Initiation was : ", rt)

        QWidget.__init__(self, *args)

        self.df = pd.DataFrame()
        self.tdf = Tdf
        self.drawtableactive = False

        self.initTable()
        self.initButtons()
        self.changeListener()

        """
        Layout for the entire View
        """
        layout = QVBoxLayout()
        layout.addWidget(self.buttons)
        layout.addWidget(self.table)

        self.setLayout(layout)
        self.resize(1280, 720)

    def initTable(self):
        """
        initializes Table
        """
        self.tablefile_loaded = False
        self.loaded_table = ""
        self.table = QTableWidget()
        self.table.setRowCount(0)
        self.table.setSortingEnabled(True)
        self.header = ['Group', 'Fraction',
                       'Spectra Filepath', 'Label', 'Sample']
        self.table.setColumnCount(len(self.header))
        self.table.setHorizontalHeaderLabels(self.header)
        self.header = self.table.horizontalHeader()

        for col in range(len(self.header)):
            if col != 2:
                self.header.setSectionResizeMode(col,
                                                 QHeaderView.ResizeToContents)
            else:
                self.header.setSectionResizeMode(col, QHeaderView.Stretch)

    def initButtons(self):
        """
        initializes Buttons
        """
        self.buttons = QWidget()
        self.textbox = QLineEdit(self)
        self.textbox.move(20, 20)
        self.textbox.setFixedHeight(20)
        self.textbox.setToolTip("Filter the experimental layout " +
                                "according to Spectra Filepath " +
                                "column. It will be dynamically \n" +
                                "updated as soon as 2 characters " +
                                "are inserted.")

        Buttons = [QPushButton('Load Project'), QPushButton('Load Table'),
                   QPushButton('Save Table'), QPushButton('Add File'),
                   QPushButton('Remove File'), QPushButton('Group'),
                   QPushButton('Fraction'), QPushButton('Label'),
                   QPushButton('Select All')]

        # Buttonlayout
        buttonlayout = QHBoxLayout()
        for button in Buttons:
            buttonlayout.addWidget(button)

        buttonlayout.addWidget(self.textbox)
        self.buttons.setLayout(buttonlayout)

        # Connections for Buttons and their apropriate functions
        Buttons[0].clicked.connect(self.loadBtnFn)
        Buttons[0].setToolTip("Load a directory with .mzML files to " +
                              "generate your own eperimental layout. " +
                              "For mzML filenames, \"F\" is the regular \n" +
                              "expression for fraction, while \"G\" or " +
                              "\"FG\"is the regular expression for the " +
                              "fraction groups.")
        Buttons[1].clicked.connect(self.importBtn)
        Buttons[1].setToolTip("Load an existing experimental layout, as " +
                              ".csv or .tsv to display and modify it.")
        Buttons[2].clicked.connect(self.exportBtn)
        Buttons[2].setToolTip("Save the experimental layout as .csv or " +
                              ".tsv file. .csv is the default option")
        Buttons[3].clicked.connect(self.loadFile)
        Buttons[3].setToolTip("Load an additional single .mzML file " +
                              "to the experimental layout.")
        Buttons[4].clicked.connect(self.RemoveBtn)
        Buttons[4].setToolTip("Remove one or more selected .mzML " +
                              "files from the experimental layout.")
        Buttons[5].clicked.connect(self.GroupBtn)
        Buttons[5].setToolTip("Set the fraction group of selected rows " +
                              "to a given number.")
        Buttons[6].clicked.connect(self.FractionBtn)
        Buttons[6].setToolTip("Set the fraction of selected rows to a " +
                              "specific number or use a range to define " +
                              "multiple fractions. This function is \n" +
                              "also able to work over multiple fraction " +
                              "groups and sets the group according to the " +
                              "fraction number.")
        Buttons[7].clicked.connect(self.LabelBtn)
        Buttons[7].setToolTip("Set the number of labels, the program will " +
                              "generate the necessary rows and will also " +
                              "define the samplenumber for you. You can \n" +
                              "apply the option to continue samplenumbers " +
                              "over mutliple fraction groups to combine " +
                              "two sample preparations.")
        Buttons[8].clicked.connect(self.SelectAllBtn)

        # init changelistener on textbox
        self.textbox.textChanged[str].connect(self.filterTable)

    def getDataFrame(self):
        return self.tdf.getTable(self)

    def drawTable(self):
        """
        draws a table with the dataframe table model in tableDataFrame
        """
        self.drawtableactive = True

        tabledf = Tdf.getTable(self)
        # print(tabledf)  # For debugging
        rowcount = len(tabledf.index)
        colcount = len(tabledf.columns)
        self.table.setRowCount(rowcount)
        for r in range(rowcount):
            row = tabledf.index[r]
            for c in range(colcount):
                col = tabledf.columns[c]
                if col == 'Spectra_Filepath':
                    path = tabledf.at[row, col].split("/")
                    name = path[len(path)-1]
                    self.table.setItem(r, c, QTableWidgetItem(name))
                else:
                    item = str(tabledf.at[row, col])
                    self.table.setItem(r, c, QTableWidgetItem(item))

        self.drawtableactive = False

    def importBtn(self):
        """
        Imports table files, currently working are csv and tsv
        """
        options = QFileDialog.Options()
        file, _ = QFileDialog.getOpenFileName(
            self, "QFileDialog.getOpenFileName()", "",
            "All Files (*);;tsv (*.tsv);; csv (*.csv)", options=options)
        
        self.path = file
        
        if file:
            df = fh.importTable(self, file)
            Tdf.setTable(self, df)
            self.drawTable()
            self.tablefile_loaded = True
            file = file.split("/")[-1]
            self.loaded_table = file

    def exportBtn(self):
        """
        Exports the table to csv or tsv;default is csv
        """
        options = QFileDialog.Options()
        file, _ = QFileDialog.getSaveFileName(
            self, "QFileDialog.getSaveFileName()", "",
            "All Files (*);;tsv (*.tsv);; csv (*.csv)", options=options)

        if file:
            self.tablefile_loaded = True
            fpath = file.split("/")[-1]
            self.loaded_table = fpath
            df = Tdf.getTable(self)
            temp = file.split("/")
            fileName = temp[len(temp)-1]
            length = len(fileName)
            if length < 4:
                ftype = "csv"
                file = file + ".csv"
            elif fileName.find('.csv', length-4) != -1:
                ftype = "csv"
            elif fileName.find('.tsv', length-4) != -1:
                ftype = "tsv"
            else:
                ftype = "csv"
                file = file + ".csv"

            fh.exportTable(self, df, file, ftype)

    def loadBtnFn(self):
        """
        provides a dialog to get the path for a directory
        and load the directory into the table.
        """
        dlg = QFileDialog(self)
        filePath = dlg.getExistingDirectory()

        if filePath != '':
            self.loadDir(filePath)

    def loadDir(self, filepath: str):
        Files = fh.getFiles(self, filepath)
        delimiters = ["_"]
        preparedFiles = fh.tagfiles(self, sorted(Files), delimiters[0])
        rawTable = fh.createRawTable(self, preparedFiles, filepath)
        Tdf.setTable(self, rawTable)
        self.drawTable()

    def loadFile(self, file: str = ""):
        """
        provides a filedialog to load an additional file to the dataframe
        """
        options = QFileDialog.Options()
        if not file:
            file, _ = QFileDialog.getOpenFileName(
                self, "QFileDialog.getOpenFileName()", "",
                "All Files (*);;mzML Files (*.mzML)", options=options)

        if file:
            cdf = Tdf.getTable(self)
            filelist = []
            filePath = file.rsplit("/", 1)[0]
            temp = file.split("/")
            fileName = temp[len(temp)-1]
            if file:
                # print(file)
                filelist.append(fileName)
                tagged_file = fh.tagfiles(self, filelist)
                df = fh.createRawTable(self, tagged_file, filePath)

                ndf = cdf.append(df, ignore_index=True)

                Tdf.setTable(self, ndf)
                self.drawTable()
            else:
                return False

    def getSelRows(self) -> list:
        """
        Function which returns a list of the Indexes of selected Rows
        todo: needs to be adjusted to fit the datamodel:
        so far index of table is only matching index of dataframe
        in first iteration, as soon as remove is called twice it crashes.
        """
        selindexes = self.table.selectionModel().selectedRows()
        selrows = []
        for index in sorted(selindexes):
            row = index.row()
            selrows.append(row)
        return selrows

    def GroupBtn(self):
        """
        Enables the user to change the group of selected rows to a given
        number.
        """
        selrows = self.getSelRows()

        groupnum, ok = QInputDialog.getInt(self,
                                           "Group Number",
                                           "Enter Integer Groupnumber")

        if ok:
            Tdf.modifyGroup(self, selrows, groupnum)
            self.drawTable()

    def RemoveBtn(self):
        """
        Enables the user to remove selected rows
        """
        selrows = self.getSelRows()
        Tdf.rmvRow(self, selrows)
        self.drawTable()

    def FractionBtn(self):
        """
        Enables the user to change the Fraction of selected rows to a given
        number or give a range.
        """
        selrows = self.getSelRows()

        # first inputdialog
        fracmin, ok = QInputDialog.getInt(self,
                                          "Fraction",
                                          "Enter minimal Fractionnumber " +
                                          "or single Fractionnumber")
        # second inputdialog if first is accepted
        if ok:
            fracmax, ok = QInputDialog.getInt(self,
                                              "Fraction",
                                              "Enter maximal Fractionnumber " +
                                              "or 0 for single Fractionnumber")
            if ok:
                # decision if multiple fractions are set or just one
                if fracmax != 0:
                    if fracmax > fracmin:
                        # third messagedialog
                        rep = QMessageBox.question(self, "Fraction Group?",
                                                   "Do you want to infer a " +
                                                   "Fraction Group from the " +
                                                   "given range?",
                                                   (QMessageBox.Yes |
                                                    QMessageBox.No),
                                                   QMessageBox.No)

                        # when confirmed the fraction froup is set
                        # when max fraction is reached.
                        if rep == QMessageBox.Yes:
                            Tdf.modifyFraction(self, selrows, fracmin, fracmax)
                            fractions = fracmax-fracmin + 1
                            numgroups = math.ceil(len(selrows)/fractions)
                            splicelist = [0]
                            for g in range(1, numgroups+1):
                                splicelist.append(g*fractions)
                            splicelist.append(len(selrows))
                            for group in range(1, numgroups+1):
                                indexa = splicelist[group-1]
                                indexb = splicelist[group]
                                subrows = selrows[indexa:indexb]
                                Tdf.modifyGroup(self, subrows, group)
                        else:
                            Tdf.modifyFraction(self, selrows, fracmin, fracmax)

                    elif fracmax == fracmin:
                        Tdf.modifyFraction(self, selrows, fracmin)

                    else:
                        QMessageBox.warning(self, "Error", "Please use " +
                                            "a higher integer " +
                                            "number for the maximum " +
                                            "fractionnumber.")

                else:
                    Tdf.modifyFraction(self, selrows, fracmin)
                self.drawTable()

    def LabelBtn(self):
        """
        Let the user choose the number of labels, it will generate
        the labels for the copied rows and also links the sample to
        the label. Gives an option to continue the samplecount
        over fraction groups.
        """
        labelnum, ok = QInputDialog.getInt(self, "Label",
                                           "Please specify the multiplicity " +
                                           "of the selected rows")
        if ok:
            rep = QMessageBox.question(self, "Continuous Sample",
                                       "Does the samplenumber " +
                                       "continue over multiple " +
                                       "fraction groups?",
                                       (QMessageBox.Yes |
                                        QMessageBox.No),
                                       QMessageBox.No)
            if rep == QMessageBox.Yes:
                try:
                    Tdf.modifyLabelSample(self, labelnum, True)
                except ValueError:
                    QMessageBox.about(self, "Warning", "Unfortunaly, " +
                                            "your Number was <1")
            else:
                try:
                    Tdf.modifyLabelSample(self, labelnum, False)
                except ValueError:
                    QMessageBox.about(self, "Warning", "Unfortunaly, " +
                                            "your Number was <1")
            self.drawTable()

    def SelectAllBtn(self):
        """
        Selects all Rows of the Table
        """
        self.table.setSelectionMode(QAbstractItemView.MultiSelection)

        for i in range(self.table.rowCount()):
            selected = self.getSelRows()

            for j in range(len(selected)):
                if i == selected[j]:
                    self.table.selectRow(i)

            self.table.selectRow(i)

        self.table.setSelectionMode(QAbstractItemView.ExtendedSelection)

    def updateTableView(self, rows):
        tabledf = Tdf.getTable(self)
        rowcount = len(tabledf.index)
        for i in range(rowcount):
            self.table.setRowHidden(i, True)
        for i in rows:
            self.table.setRowHidden(i, False)

    def filterTable(self):
        """
        get changes from textbox and update table when
        more than 3 characters are given.
        then update table with the rows that
        contain the input in the give column.
        """
        tb = self.textbox
        givencolumn = "Spectra_Filepath"
        tbinput = tb.text()
        ft = Tdf.getTable(self)
        validDf = not(ft.empty or ft.dropna().empty)
        # print(validDf)  # for debugging
        # print(type(ft))  # for debugging
        if len(tbinput) >= 2:
            rowstoshow = ft[ft[givencolumn].str.contains(tbinput)]
            self.updateTableView(rowstoshow.index)
        else:
            self.updateTableView(ft.index)

    def changeListener(self):
        self.table.itemChanged.connect(self.editField)

    def editField(self):
        if len(self.table.selectedItems()) == 1:
            if not self.drawtableactive:
                itemchanged = self.table.currentItem()
                newvalue = itemchanged.text()
                row = itemchanged.row()
                column = itemchanged.column()
                if column != 2:
                    Tdf.modifyField(self, row, column, newvalue)
                    self.drawTable()
                else:
                    QMessageBox.about(self, "Warning", "Please only, " +
                                            "modify attribute columns," +
                                            "not the filepath.\n" +
                                            "To change the filepath," +
                                            "use remove and add file.")
                    self.drawTable()



import sys
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import (QWidget, QToolTip,
                             QPushButton, QApplication, QMainWindow,
                             QAction, qApp,
                             QHBoxLayout, QVBoxLayout, QMessageBox,
                             QLineEdit, QTableWidget, QTableWidgetItem,
                             QGridLayout, QScrollArea, QPlainTextEdit,
                             QDesktopWidget, QLabel, QRadioButton,
                             QGroupBox, QSizePolicy, QCheckBox, QFileDialog,
                             QTextEdit, QTextBrowser, QInputDialog)
from PyQt5.QtGui import QFont, QColor, QTextCharFormat, QTextCursor
from PyQt5.QtCore import Qt, QUrl
# from dictionaries import Dict
sys.path.insert(0, '../examples')
from LoadFasta_FastaViewer import LoadFasta_FastaViewer  # NOQA: E402
from FilesNumberHandler import Files_Number_Handler



class GUI_FastaViewer(QMainWindow):
    """
    A class used to make and change the appearance of the FastaViewer.
    It enables to load a fasta file of a colletion of protein sequences
    and search for proteins by its accesion number, name or subsequence.
    ...
    Attributes
    ----------
    searchButtonP : QtWidgets.QPushButton
        a button to be shown on window and exc action on click
    searchButtonP : QtWidgets.QPushButton
         Button to be shown on window and exc action on click
    boxPro : QLineEdit(self)
         Textfield in wich the user can type inputs
    tw : QtWidgets.QTreeWidget
         Treewidget used to hold and show data on the sceen
    mainwidget : QWidget
         QWidget that contains all the Widgets
    main_layout : QVBoxLayout
        The main Layout of the Window, contains all other Layouts
    set1,set2,set3 : QHBoxLayout()
         Horizontal Layouts that hold different Widgets
    radioname,radioid,radioseq=QRadioButton
            A QRadioButton that appears on sceen an can be cheked
    color : QColor
            Red Color for the searched seq
    colorblack : QColor
            Black color for the found Protein sequence
    fileloaded : int
            Integer to check if file has been loaded
    Methods
    -------
    _init_(self)
        Sets Window size na exc. initUI()
    initUi(self)
            Creates the User Interface
    center(self)
            Centers the Window on screen
    cutstring(self,oldstring,proteinseq)
            Cuts the oldstring when proteinseq appears and returns a list of
            the cuts
    loadFile(self)
            A function for the loadbutton that open QFileDialog and saves the
            Path to the file fo the logic class
    searchClicked(self)
            A function for the searchButtonP, it checks if input is correct
            and exc a search function
    radioIdSearch()
            Searches for the Protein Information by ID
    sequenceSearch()
            Searches for the Protein Information by sequence and also changes
            the color of the are of the sequence that is the same as the input
    nameSearch()
            Searches for the Protein Information by name
    main()
            runs the QApplication
    """

    def __init__(self):
        """Gets self and sets Window size na exc. initUI()
        Parameters
        ----------
        self : QMainWindow
        Returns
        -------
        nothing
        """
        super().__init__()
        self.resize(1280, 720)
        self.path = ""
        self.initUI()

    def initUI(self):
        """Gets self and creates the User Interface
        Parameters
        ----------
        self : QMainWindow
        Returns
        -------
        nothing
        """

        # creating Buttons
        self.searchButtonP = QtWidgets.QPushButton(self)
        self.searchButtonP.setText("Search")
        self.searchButtonP.clicked.connect(self.searchClicked)

        self.loadbutton = QtWidgets.QPushButton(self)
        self.loadbutton.setText("Load")
        self.loadbutton.clicked.connect(self.loadPath)

        self.changeReversePattern = QtWidgets.QPushButton(self)
        self.changeReversePattern.setText("Add Reverse Pattern")
        self.changeReversePattern.clicked.connect(self.user_Dialog_ChangeReversePattern)

        # creating testboxes for the buttons
        self.boxPro = QLineEdit(self)

        # Creating treewidget for displaying the proteins
        self.tw = QtWidgets.QTreeWidget()
        self.tw.setHeaderLabels(["Accession", "Organism", "Protein Name"])
        self.tw.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        # Layout
        self.mainwidget = QWidget(self)
        self.main_layout = QVBoxLayout(self.mainwidget)

        # every set contains all widgets on the level they should be in the UI
        # in set1 there is a textbox and a button
        self.set1 = QHBoxLayout()
        self.set1.addWidget(self.boxPro)
        self.set1.addWidget(self.searchButtonP)
        self.set1.addWidget(self.loadbutton)

        # set 2 contains the radiobuttons and a label
        self.set2 = QHBoxLayout()
        self.radioname = QRadioButton("Name")
        self.radioid = QRadioButton("ID")
        self.radioseq = QRadioButton("Sequence")
        self.radioname.setChecked(True)
        self.decoycheck = QCheckBox("Decoy search", self) 
        self.datalabel = QLabel()
        self.datalabel.setText("Data not loaded")
        self.set2.addWidget(self.radioname)
        self.set2.addWidget(self.radioid)
        self.set2.addWidget(self.radioseq)
        self.set2.addWidget(self.decoycheck)
        self.set2.addWidget(self.changeReversePattern)
        self.set2.addWidget(self.datalabel)

        # set 3 contains the table and the result box
        self.set3 = QHBoxLayout()
        self.set3.addWidget(self.tw)

        # adding all QHBoxLayout to the main QVBoxLayout
        self.main_layout.addLayout(self.set1)
        self.main_layout.addLayout(self.set2)
        self.main_layout.addLayout(self.set3)

        self.mainwidget.setLayout(self.main_layout)
        self.setCentralWidget(self.mainwidget)
        self.setWindowTitle('Protein Viewer')

        self.extra_pattern = ''

        # defining some colors to marked searched sequences
        self.color = QColor(255, 0, 0)
        self.colorblack = QColor(0, 0, 0)
        self.center()
        self.show()
        # defining a counter var that checks if a file has been loaded
        # if not than there should be an error Window when a search is done

        self.fileloaded = 0

        # centering the widget
    def center(self):
        """Gets self and centers the Window
        Parameters
        ----------
        self : QMainWindow
        Returns
        -------
        changes so that the Window appears on the center of the screen
        """
        qr = self.frameGeometry()
        cp = QDesktopWidget().availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

    # defining a help function to cut the sequence that is being search from the
    # Protein sequence that has been found

    def cutstring(self, oldstring: str, proteinseq: str) -> str:
        """Gets two strings and splits the first string in a list
            on the playces where the second string is found. This helps
            to change the color of the sequences later on
    Parameters
    ----------
    oldstring : str
        the enteire proteinseq as a string
    proteinseq : str
        is the searched seq and is used to split the entire seq in parts
    Returns
    -------
    list
        a list of strings to be later put together after recoloring
    """
        cut = oldstring.split(proteinseq)
        return cut
    # defining the function for load button to get path of database
    def clearFastaViewer(self):
        self.tw.clear()

    def loadPath(self):
        self.filename = QFileDialog.getOpenFileName()
        #saving the file path in the dictionary
        Files_Number_Handler.Dictionary_Change_File("fasta", self.filename[0])
        Files_Number_Handler.Dictionary_Change_Boolean("fasta")
        self.loadFile(self.filename[0])


    # creates a user dialog to change default reverse pattern 
    def user_Dialog_ChangeReversePattern(self):
        text, okPressed = QInputDialog.getText(self, "Add reverse pattern", "Add:", QLineEdit.Normal, "")
        if okPressed and text != '':
            self.extra_pattern = text
    

    def option_selected(self,button):
        global Option_selected
        Option_selected = button.text()


    def loadFile(self, fasta_path):
        """Gets QMainWindow and opens a QFileDialog and loads path
    Parameters
    ----------
    self : QMainWindow
        the MainWindow of the class
    Returns
    -------
    nothing , it changes the QMainWindow so that the user can see that a file
    has been loaded
    """
        #self.filename = QFileDialog.getOpenFileName()
        self.path = fasta_path
        self.fileloaded = 1
        # loading the lists before searching in order to make the search faster
        self.dictKeyAccession, self.proteinList, self.proteinNameList, self.proteinOSList, self.dictKeyAccessionDECOY, self.proteinListDECOY, self.proteinNameListDECOY, self.proteinOSListDECOY = LoadFasta_FastaViewer.protein_dictionary(
            fasta_path, self.extra_pattern)
        self.datalabel.setText("Data loaded")
        for i in range(len(self.dictKeyAccession)):
            ID = list(self.dictKeyAccession.keys())[i]
            Proteinname = self.proteinNameList[i]
            OS = self.proteinOSList[i]
            self.cg = QtWidgets.QTreeWidgetItem(self.tw)
            self.cg.setData(0, 0, ID)
            self.cg.setData(1, 0, OS)
            self.cg.setData(2, 0, Proteinname)
        self.tw.itemClicked.connect(self.clickTreeItem)

    # defining a function to creat TreeItems

    def createTreeItem(self, item, ID, Protein):
        """Gets a TreeItem and creats two child Items, a Qlabel and a QTextEdit.
    Firs Child Item holds a QTextEdit with the given Protein sequence.
    Second Cild Item holds a QLabel with a hyperlink to the database UniProt.
    Parameters
    ----------
    self : QMainWindow
        the MainWindow of the class
    item : QTreeWidgetItem
        for which the child items will be created
    ID : The specific protein accesion
        which is needed for the link to the database
    Protein : The specific protein sequence
        that will be displayed in the QTextEdit
    Returns
    -------
    nothing , it changes the Treewidget
    and creates two child items for the handed in tree item
    """
        self.link = QLabel()
        self.link.setTextInteractionFlags(
            Qt.LinksAccessibleByMouse)
        self.link.setOpenExternalLinks(True)
        self.link.setTextFormat(Qt.RichText)
        self.link.setText("<a href =" + "https://www.uniprot.org/uniprot/" +
                          ID + ">" + "More Information"+" </a>")
        self.textp = QTextEdit()
        self.textp.resize(
            self.textp.width(), self.textp.height())
        self.textp.insertPlainText(
            "Proteinsequence: " + Protein + "\n")
        self.textp.setReadOnly(True)
        self.cgChild = QtWidgets.QTreeWidgetItem(
            item)
        self.cgChild2 = QtWidgets.QTreeWidgetItem(
            item)
        self.cgChild.setFirstColumnSpanned(True)
        self.tw.setItemWidget(
            self.cgChild, 0, self.textp)
        self.tw.setItemWidget(
            self.cgChild2, 0, self.link)

    # methode when TreeItem was cklicked

    def clickTreeItem(self, item):
        '''Gets a QTreeWidgetItem and its ID data of the first
        collumn. The ID and the corresponding protein sequence are
        handed to the createTreeItem method.
        Parameters
        ----------
        self : QMainWindow
            the MainWindow of the class
        item : clicked QTreeWidgetItem
            from which the ID is obtained
        Returns
        -------
        nothing
        '''
        num = item.childCount()
        # prevents multiple creation of the same child tree items
        if num == 0:
            ID = item.data(0, 0)
            index = list(self.dictKeyAccession.keys()).index(ID)
            Protein = self.proteinList[index]
            self.createTreeItem(item, ID, Protein)

    def clickTreeItemDecoy(self, item):
        '''Does the same as clickTreeItem but
        hands the corresponding DECOY protein sequence
        to the create TreeItem method.
        '''
        num = item.childCount()
        if num == 0:
            ID = item.data(0, 0)
            index = list(self.dictKeyAccessionDECOY).index(ID)
            Protein = self.proteinListDECOY[index]
            self.createTreeItem(item, ID, Protein)

    def createTreeItemSeqSearch(self, item, ID, Protein):
        """Gets a TreeItem and creats two child Items and a Qlabel.
        Firs Child Item holds a QTextEdit with the given Protein sequence.
        Second Cild Item holds a QLabel with a hyperlink to the database UniProt.
        Parameters
        ----------
        self : QMainWindow
            the MainWindow of the class
        item : QTreeWidgetItem
            for which the child items will be created
        ID : The specific protein accesion
            which is needed for the link to the database
        Protein : A QTextEdit widget with the specific portein sequence
        Returns
        -------
        nothing , it changes the Treewidget
        and creates two child items for the handed in tree item
        """
        self.link = QLabel()
        self.link.setTextInteractionFlags(
            Qt.LinksAccessibleByMouse)
        self.link.setOpenExternalLinks(True)
        self.link.setTextFormat(Qt.RichText)
        self.link.setText("<a href =" + "https://www.uniprot.org/uniprot/" +
                          ID + ">" + "More Information"+" </a>")

        self.cgChild = QtWidgets.QTreeWidgetItem(
            item)
        self.cgChild2 = QtWidgets.QTreeWidgetItem(
            item)
        self.cgChild.setFirstColumnSpanned(True)
        self.tw.setItemWidget(
            self.cgChild, 0, Protein)
        self.tw.setItemWidget(
            self.cgChild2, 0, self.link)

    def clickTreeItemSeqSearch(self, item):
        '''Gets a QTreeWidgetItem and its ID data of the first
        collumn. The ID and the corresponding QTextEdit widget with the
        protein sequence are handed to the createTreeItem method.
        Parameters
        ----------
        self : QMainWindow
            the MainWindow of the class
        item : clicked QTreeWidgetItem
            from which the ID is obtained
        Returns
        -------
        nothing
        '''
        num = item.childCount()
        if num == 0:
            ID = item.data(0, 0)
            Protein = self.SequencSearchDict.get(ID)
            self.createTreeItemSeqSearch(item, ID, Protein)

    def clickTreeItemSeqSearchDecoy(self, item):
        '''Does the same as clickTreeItemSeqSearch but
        hands the corresponding DECOY protein sequence
        to the create TreeItem method.
        '''
        num = item.childCount()
        if num == 0:
            ID = item.data(0, 0)
            Protein = self.SequencSearchDictDECOY.get(ID)
            self.createTreeItemSeqSearch(item, ID, Protein)

    # defining the searchClicked method for the searchButtonP

    def searchClicked(self):
        """Gets self and searches for Protein and shows the result
        on QMainWindow
    Parameters
    ----------
    self : QMainWindow
    Returns
    -------
    nothing but changes the QMainWindow to show the Protein or an Error message
    """
        if self.fileloaded == 0:
            self.error = QMessageBox()
            self.error.setIcon(QMessageBox.Information)
            self.error.setText("Please load Data before searching")
            self.error.setWindowTitle("Error")
            c = self.error.exec_()
        else:
            self.tw.clear()
            # check if inputbox is empty. if empty return error if not proceed
            if self.boxPro.text() == "":
                self.error = QMessageBox()
                self.error.setIcon(QMessageBox.Information)
                self.error.setText("Please enter input before searching")
                self.error.setWindowTitle("Error")
                a = self.error.exec_()
            else:
                if self.radioid.isChecked():
                    self.radioIdSearch()

                if self.radioseq.isChecked():
                    self.sequenceSearch()

                if self.radioname.isChecked():
                    self.nameSearch()
        # doc recommends enabling sorting after loading the tree with elements
        self.tw.setSortingEnabled(True)

    def radioIdSearch(self):
        """Gets self and searches for Protein based on ID
            and shows the result on QMainWindow, also adds a hyperlink for
            more Information
        Parameters
        ----------
        self : QMainWindow
        Returns
        -------
        nothing but changes the QMainWindow to show the Protein in treewidget
        """
        atLeastOneProteinFound = False
        protein_accession_maybe_sub_sequence = self.boxPro.text()

        if self.decoycheck.isChecked():
            for protein_accession in self.dictKeyAccessionDECOY:
                if protein_accession_maybe_sub_sequence in protein_accession:
                    atLeastOneProteinFound = True
                    index = list(self.dictKeyAccessionDECOY).index(
                        protein_accession)
                    ID = list(self.dictKeyAccessionDECOY.keys())[
                        index]
                    Proteinname = self.proteinNameListDECOY[index]
                    OS = self.proteinOSListDECOY[index]
                    self.cg = QtWidgets.QTreeWidgetItem(self.tw)
                    self.cg.setData(0, 0, ID)
                    self.cg.setData(1, 0, OS)
                    self.cg.setData(2, 0, Proteinname)

            header = self.tw.header()
            header.setSectionResizeMode(
                QtWidgets.QHeaderView.ResizeToContents)
            header.setStretchLastSection(True)
            self.tw.itemClicked.disconnect()
            self.tw.itemClicked.connect(self.clickTreeItemDecoy)

        else:
            for protein_accession in self.dictKeyAccession:
                if protein_accession_maybe_sub_sequence in protein_accession:
                    atLeastOneProteinFound = True
                    index = list(self.dictKeyAccession).index(
                        protein_accession)
                    ID = list(self.dictKeyAccession.keys())[index]
                    Proteinname = self.proteinNameList[index]
                    OS = self.proteinOSList[index]
                    self.dummy = ID
                    self.cg = QtWidgets.QTreeWidgetItem(self.tw)
                    self.cg.setData(0, 0, ID)
                    self.cg.setData(1, 0, OS)
                    self.cg.setData(2, 0, Proteinname)

            header = self.tw.header()
            header.setSectionResizeMode(
                QtWidgets.QHeaderView.ResizeToContents)
            header.setStretchLastSection(True)
            self.tw.itemClicked.disconnect()
            self.tw.itemClicked.connect(self.clickTreeItem)

        if not atLeastOneProteinFound:
            self.msg = QMessageBox()
            self.msg.setIcon(QMessageBox.Information)
            self.msg.setText(
                "No matching protein accession found in database.")
            self.msg.setWindowTitle("Error")
            x = self.msg.exec_()

    def sequenceSearch(self):
        """Gets self and searches for Protein based on sequence
            and shows the result on QMainWindow, also adds a hyperlink for
            more Information
        Parameters
        ----------
        self : QMainWindow
        Returns
        -------
        nothing but changes the QMainWindow to show the Protein in treewidget
        also changes the color of the parts of the sequence
        that are being searched
        """
        atLeastOneProteinFound = False
        protein_sub_sequence = self.boxPro.text()
        # dictionaries with ID as key and corresponding QTextEdit with protein sequence as value
        self.SequencSearchDict = {}
        self.SequencSearchDictDECOY = {}

        if self.decoycheck.isChecked():
            for protein_sequence in self.proteinListDECOY:
                if protein_sub_sequence in protein_sequence:
                    atLeastOneProteinFound = True
                    index = self.proteinListDECOY.index(
                        protein_sequence)
                    ID = list(self.dictKeyAccessionDECOY.keys())[
                        index]
                    Protein = self.proteinListDECOY[index]
                    Proteinname = self.proteinNameListDECOY[index]
                    OS = self.proteinOSListDECOY[index]

                    self.cg = QtWidgets.QTreeWidgetItem(self.tw)
                    self.cg.setData(0, 0, ID)
                    self.cg.setData(1, 0, OS)
                    self.cg.setData(2, 0, Proteinname)

                    self.textp = QTextEdit()
                    self.textp.resize(
                        self.textp.width(), self.textp.height())
                    cuts = self.cutstring(
                        Protein, protein_sub_sequence)
                    self.textp.insertPlainText(
                        "Proteinsequence: "
                    )

                    for i in range(len(cuts)):
                        # sofern wir ganz am Anfang der Liste sind
                        if (cuts[i] == '' and i == 0):
                            self.textp.setTextColor(self.color)
                            self.textp.insertPlainText(
                                protein_sub_sequence)
                            self.textp.setTextColor(
                                self.colorblack)
                        # sofern wir mitten drin sind und der sub_string mehrfach auftaucht
                        elif (cuts[i] == ''):
                            self.textp.setTextColor(self.color)
                            self.textp.insertPlainText(
                                protein_sub_sequence)
                            self.textp.setTextColor(
                                self.colorblack)
                        else:
                            if (i == len(cuts) - 1):
                                self.textp.insertPlainText(cuts[i])
                            else:
                                self.textp.insertPlainText(cuts[i])
                                self.textp.setTextColor(self.color)
                                self.textp.insertPlainText(
                                    protein_sub_sequence)
                                self.textp.setTextColor(
                                    self.colorblack)
                    self.textp.insertPlainText("\n")

                    self.textp.setReadOnly(True)
                    self.SequencSearchDictDECOY[ID] = self.textp

            header = self.tw.header()
            header.setSectionResizeMode(
                QtWidgets.QHeaderView.ResizeToContents)
            header.setStretchLastSection(True)
            self.tw.itemClicked.disconnect()
            self.tw.itemClicked.connect(self.clickTreeItemSeqSearchDecoy)

        else:
            for protein_sequence in self.proteinList:
                if protein_sub_sequence in protein_sequence:
                    atLeastOneProteinFound = True
                    index = self.proteinList.index(
                        protein_sequence)
                    ID = list(self.dictKeyAccession.keys())[index]
                    Protein = self.proteinList[index]
                    Proteinname = self.proteinNameList[index]
                    OS = self.proteinOSList[index]

                    self.cg = QtWidgets.QTreeWidgetItem(self.tw)
                    self.cg.setData(0, 0, ID)
                    self.cg.setData(1, 0, OS)
                    self.cg.setData(2, 0, Proteinname)

                    self.textp = QTextEdit()
                    self.textp.resize(
                        self.textp.width(), self.textp.height())
                    cuts = self.cutstring(
                        Protein, protein_sub_sequence)
                    self.textp.insertPlainText(
                        "Proteinsequence: "
                    )

                    for i in range(len(cuts)):
                        # sofern wir ganz am Anfang der Liste sind
                        if (cuts[i] == '' and i == 0):
                            self.textp.setTextColor(self.color)
                            self.textp.insertPlainText(
                                protein_sub_sequence)
                            self.textp.setTextColor(
                                self.colorblack)
                        # sofern wir mitten drin oder am Ende sind sind
                        elif (cuts[i] == ''):
                            self.textp.setTextColor(self.color)
                            self.textp.insertPlainText(
                                protein_sub_sequence)
                            self.textp.setTextColor(
                                self.colorblack)
                        else:
                            if (i == len(cuts) - 1):
                                self.textp.insertPlainText(cuts[i])
                            else:
                                self.textp.insertPlainText(cuts[i])
                                self.textp.setTextColor(self.color)
                                self.textp.insertPlainText(
                                    protein_sub_sequence)
                                self.textp.setTextColor(
                                    self.colorblack)

                    self.textp.setReadOnly(True)
                    self.SequencSearchDict[ID] = self.textp

            header = self.tw.header()
            header.setSectionResizeMode(
                QtWidgets.QHeaderView.ResizeToContents)
            header.setStretchLastSection(True)
            self.tw.itemClicked.disconnect()
            self.tw.itemClicked.connect(self.clickTreeItemSeqSearch)

        if not atLeastOneProteinFound:
            self.msg = QMessageBox()
            self.msg.setIcon(QMessageBox.Information)
            self.msg.setText(
                "No matching protein sequence found in database.")
            self.msg.setWindowTitle("Error")
            x = self.msg.exec_()

    def nameSearch(self):
        """Gets self and searches for Protein based on Proteinname
            and shows the result on QMainWindow, also adds a hyperlink for
            more Information
        Parameters
        ----------
        self : QMainWindow
        Returns
        -------
        nothing but changes the QMainWindow to show the Protein in treewidget
        """
        atLeastOneProteinFound = False
        protein_sub_name = self.boxPro.text()

        if self.decoycheck.isChecked():
            for protein_name in self.proteinNameListDECOY:
                if protein_sub_name in protein_name:
                    atLeastOneProteinFound = True
                    index = self.proteinNameListDECOY.index(
                        protein_name)
                    ID = list(self.dictKeyAccessionDECOY.keys())[
                        index]
                    Proteinname = protein_name
                    OS = self.proteinOSListDECOY[index]

                    self.cg = QtWidgets.QTreeWidgetItem(self.tw)
                    self.cg.setData(0, 0, ID)
                    self.cg.setData(1, 0, OS)
                    self.cg.setData(2, 0, Proteinname)

            header = self.tw.header()
            header.setSectionResizeMode(
                QtWidgets.QHeaderView.ResizeToContents)
            header.setStretchLastSection(True)
            self.tw.itemClicked.disconnect()
            self.tw.itemClicked.connect(self.clickTreeItemDecoy)

        else:
            for protein_name in self.proteinNameList:
                if protein_sub_name in protein_name:
                    atLeastOneProteinFound = True
                    index = self.proteinNameList.index(
                        protein_name)
                    ID = list(self.dictKeyAccession.keys())[index]
                    Proteinname = self.proteinNameList[index]
                    OS = self.proteinOSList[index]

                    self.cg = QtWidgets.QTreeWidgetItem(self.tw,)
                    self.cg.setData(0, 0, ID)
                    self.cg.setData(1, 0, OS)
                    self.cg.setData(2, 0, Proteinname)

            header = self.tw.header()
            header.setSectionResizeMode(
                QtWidgets.QHeaderView.ResizeToContents)
            header.setStretchLastSection(True)
            self.tw.itemClicked.disconnect()
            self.tw.itemClicked.connect(self.clickTreeItem)

        if not atLeastOneProteinFound:
            self.msg = QMessageBox()
            self.msg.setIcon(QMessageBox.Information)
            self.msg.setText(
                "No matching protein name found in database.")
            self.msg.setWindowTitle("Error")
            x = self.msg.exec_()


def main():
    """Gets nothing, runs the QApplication
    Parameters
    ----------
    nothing
    Returns
    -------
    nothing
    """
    app = QApplication(sys.argv)
    ex = GUI_FastaViewer()
    sys.exit(app.exec_())

"""
if __name__ == '__main__':
    main()
"""



import os
import sys
import glob
import pandas as pd
import re

files = []


class FileHandler:
    """
    Filehandler class that provides support for file and directory interaction.
    """
    def __init__(self, path: str, files: list, delimiter: str):
        self.path = path
        self.files = files
        self.delimiter = delimiter

    def getFiles(self, path: str) -> list:
        """
        Scans a provided directory and returns a list of all
        the mzML files.
        """
        os.chdir(path)
        files = glob.glob('*.mzML')
        return files

    def tagfiles(self, files: list, delimiter: str = "_"):
        """
        Parses filenames in a list of files by a provided delimiter.
        Default delimiter is _ .
        Returns a list of tags.
        """
        tagproperty = {}
        for file in files:
            name = file
            tags = []
            file = file.split(".")[0]
            tags = file.split(delimiter)
            tagproperty[name] = tags
        return tagproperty

    def importTable(self, file: str) -> pd.DataFrame:
        """
        Imports a csv or tsv file and returns a panda-dataframe.
        Returns false if filetype is not tsv or csv.
        """
        ftype = file.split(".")[1]

        if ftype == "csv":
            fimport = pd.read_csv(file)

        elif ftype == "tsv":
            fimport = pd.read_csv(file, sep='\t')
        else:
            return False
        return fimport

    def exportTable(self, table: pd.DataFrame,
                    filename: str, ftype: str = 'csv') -> bool:
        """
        Exports a panda-dataframe to the specified filetype.
        builds a path from a provided filename, a provided path,
        and the provided filetype.
        Defaults:
        If no path is specified, default is the current working directory.
        If no filetype is specified, default is csv.
        Returns:
        Returns false if specified filetype is not supported.
        Returns true if file was successfully written.
        Returns an error if writing the file failed.
        """

        fullpath = filename
        encodingOption = 'utf-8'
        if ftype == 'tsv':
            separator = "\t"
        elif ftype == 'csv':
            separator = ","
        else:
            return False
        try:
            with open(fullpath, 'w') as fileToWrite:
                fileToWrite.write(table.to_csv(
                    sep=separator, index=False, encoding=encodingOption))
            return True
        except:
            e = sys.exc_info()[0]
            return e

    def createRawTable(self, tagdict: dict, inputdir: str):
        """
        Creates pandas-dataframe from prepared files dictionary, which
        contains the filename and its tags (delimiter is _). Dataframes index
        is integer based, header are the column names. Searches the
        tags for regular expressions to automatically include fraction and
        fraction group from filename.
        Default regex:
        Fraction-Group: FG or G
        Fraction: F
        """
        columnregex = ["FG[0-9]+", "G[0-9]+", "F[0-9]+"]
        header = ['Fraction_Group', 'Fraction',
                  'Spectra_Filepath', 'Label', 'Sample']
        index = []
        rows = []
        filenames = tagdict.keys()
        for file in filenames:
            index.append(file)
            filtered_tags = {'Fraction_Group': 0, 'Fraction': 0,
                             'Spectra_Filepath': str(inputdir + "/" + file),
                             'Label': 0, 'Sample': 0}
            filetags = tagdict[file]
            for tag in filetags:
                if re.match(columnregex[0], tag):
                    filtered_tags['Fraction_Group'] = tag.split('FG')[1]
                elif re.match(columnregex[1], tag):
                    filtered_tags['Fraction_Group'] = tag.split('G')[1]
                elif re.match(columnregex[2], tag):
                    filtered_tags['Fraction'] = tag.split('F')[1]
                else:
                    continue
            rows.append([filtered_tags[i]for i in header])
        rawtable = pd.DataFrame(rows, columns=header)
        return rawtable



import re

class LoadFasta_FastaViewer:

    # Compilation for the ProteinId_dictionary and the lists with its values for each Protein
    def protein_dictionary(fastaFile, extra_pattern):
        """Gets fasta File Path and saves all informations about the proteins 
        in separate lists 
        Parameters
        ----------
        fastaFile : path to fasta file
        Returns
        -------
        for all proteins either Decoy or normal Lists/ dictionary.
        """
        dictKeyAccession = {}
        proteinList = []
        proteinNameList = []
        proteinOSList = []
        dictKeyAccessionDECOY = {}
        proteinListDECOY = []
        proteinNameListDECOY = []
        proteinOSListDECOY = []

        # if there is no accession, so we have no key
        default = 0

        #with open(fastaFile) as file_content:
        file_content = open(fastaFile)

        # Go through the fasta file, line by line
        nextline = file_content.readline()

        while(True):

            # if protein declaration (next protein) was found
            if nextline.startswith('>'):
                
                # safe information about the protein declaration for when inserting informations into lists
                protein_declaration = nextline

                check_if_reverse = protein_declaration.find('_rev')

                # check for extra pattern if used
                check_for_extra_pattern = -1
                if extra_pattern != '':
                    check_for_extra_pattern = protein_declaration.find(extra_pattern)

                # find upper and lower index of Protein Accession (ID)
                bounds = [m.start() for m in re.finditer(r'\|', protein_declaration)]

                # if a Protein Accesion (ID) was found
                if len(bounds) >= 1:

                    # if = 1, then the protein accession start at character 1 
                    if len(bounds) == 1:
                        key = (protein_declaration[1:bounds[0]])
                    else:
                        key = (protein_declaration[bounds[0]+1:bounds[1]])

                    descr_upper_index = protein_declaration.find('OS=')

                    os_upper_index = -1

                    # find out up to which index the OS goes
                    if descr_upper_index != -1:
                        # start searchin in seqs from index 'OS='
                        os_upper_index = protein_declaration.find('(', descr_upper_index)

                    os = 'not found'

                    # if no upper index for the OS was found use the line from "OS=" till end of line
                    if (os_upper_index == -1):
                        # if "OS=" was found, else stick with "os = not found"
                        if descr_upper_index != -1:
                            stringFrom_OS_TillEndOfLine = protein_declaration[descr_upper_index+3:]
                            listOfAllWordsTillEndOfLine = stringFrom_OS_TillEndOfLine.split()
                            os = listOfAllWordsTillEndOfLine[0] + ' ' + listOfAllWordsTillEndOfLine[1]

                    name = 'not found'

                    if descr_upper_index != -1:
                        if len(bounds) == 1:
                            name = (protein_declaration[bounds[0]+1:descr_upper_index])
                        else:
                            name = (protein_declaration[bounds[1]+1:descr_upper_index])
                        
                    protein_sequence_string = ""
                    nextline = file_content.readline()

                    # read file line by line, till new protein (begins with '>')
                    while not nextline.startswith('>'):
                        protein_sequence_string += nextline[:-1]
                        try:
                            nextline = next(file_content)
                        except Exception:
                            break

                    # set the values inside of the dictionary and the lists
                    # is decoy or reverse(!=-1) -> use seperate List to safe informations
                    if protein_declaration.startswith('>DECOY') or check_if_reverse != -1 or check_for_extra_pattern != -1:
                        dictKeyAccessionDECOY[key] = protein_sequence_string
                        proteinListDECOY.append(protein_sequence_string)
                        proteinNameListDECOY.append(name)

                        if (os_upper_index == -1 or descr_upper_index == -1):
                            proteinOSListDECOY.append(os)
                        else:
                            proteinOSListDECOY.append((protein_declaration[descr_upper_index+3:os_upper_index]))

                    # regular protein -> use regular list 
                    else:
                        dictKeyAccession[key] = protein_sequence_string
                        proteinList.append(protein_sequence_string)
                        proteinNameList.append(name)

                        if (os_upper_index == -1 or descr_upper_index == -1):
                            proteinOSList.append(os)
                        else:
                            proteinOSList.append((protein_declaration[descr_upper_index+3:os_upper_index]))
                    
                # if no Protein Accession (ID) was found
                else:
                    descr_upper_index = protein_declaration.find('OS=')

                    if descr_upper_index == -1:
                        key = str(default)
                        name = 'not found'
                        default = default + 1
                    else:    
                        key = protein_declaration.split("OS=")[0]
                        name = key
                    

                    os_upper_index = -1

                    # find out up to which index the OS goes
                    if descr_upper_index != -1:
                        # start searchin in seqs from index 'OS='
                        os_upper_index = protein_declaration.find('(', descr_upper_index)

                    os = 'not found'

                    # if no upper index for the OS was found use the line from "OS=" till end of line
                    if (os_upper_index == -1):
                        # if "OS=" was found, else stick with "os = not found"
                        if descr_upper_index != -1:
                            stringFrom_OS_TillEndOfLine = protein_declaration[descr_upper_index+3:]
                            listOfAllWordsTillEndOfLine = stringFrom_OS_TillEndOfLine.split()
                            os = listOfAllWordsTillEndOfLine[0] + ' ' + listOfAllWordsTillEndOfLine[1]

                    protein_sequence_string = ""
                    nextline = file_content.readline()

                    # read file line by line, till new protein (begins with '>')
                    while not nextline.startswith('>'):
                        protein_sequence_string += nextline[:-1]
                        try:
                            nextline = next(file_content)
                        except Exception:
                            break

                    # set the values inside of the dictionary and the lists
                    # is decoy -> use seperate List to safe informations
                    if protein_declaration.startswith('>DECOY') or check_if_reverse != -1 or check_for_extra_pattern != -1:
                        dictKeyAccessionDECOY[key] = protein_sequence_string
                        proteinListDECOY.append(protein_sequence_string)
                        proteinNameListDECOY.append(name)

                        if (os_upper_index == -1 or descr_upper_index == -1):
                            proteinOSListDECOY.append(os)
                        else:
                            proteinOSListDECOY.append((protein_declaration[descr_upper_index+3:os_upper_index]))

                    else:
                        dictKeyAccession[key] = protein_sequence_string
                        proteinList.append(protein_sequence_string)
                        proteinNameList.append(name)

                        if (os_upper_index == -1 or descr_upper_index == -1):
                            proteinOSList.append(os)
                        else:
                            proteinOSList.append((protein_declaration[descr_upper_index+3:os_upper_index]))
            
            else:
                break

        return dictKeyAccession, proteinList, proteinNameList, proteinOSList, dictKeyAccessionDECOY, proteinListDECOY, proteinNameListDECOY, proteinOSListDECOY         


        


def main():

    # for testing purposes only
    # use your own path for it
    dictKeyAccession, proteinList, proteinNameList, proteinOSList, dictKeyAccessionDECOY, proteinListDECOY, proteinNameListDECOY, proteinOSListDECOY = LoadFasta_FastaViewer.protein_dictionary(
        "path/to/file.fasta", "insert extra decoy pattern here")

    proteinnameSub = input("Name: ")

    for protein_name in proteinNameList:
        if proteinnameSub in protein_name:
            print("Proteinname: " + protein_name)

    # here the users enters the protein accession (key), or only a part of it
    # e.g.: 'P00761'
    protein_accession_maybe_sub_sequence = input("Bitte Protein accession angeben: ")

    # index starts with 0 (for the lists and the dictionary)
    # search for key
    for protein_accession in dictKeyAccession:
        if protein_accession_maybe_sub_sequence in protein_accession:
            index = list(dictKeyAccession).index(protein_accession)
            print("ID: " + list(dictKeyAccession.keys())[index])
            print("Protein: " + dictKeyAccession.get(protein_accession))
            print("Proteinname: " + proteinNameList[index])
            print("OS: " + proteinOSList[index])


if __name__ == "__main__":
    main()



import pyopenms
from PyQt5.QtCore import (
    Qt,
)
from PyQt5.QtWidgets import (
    QHBoxLayout,
    QWidget,
    QSplitter,
)
from ScanTableWidget import ScanTableWidget

from SpectrumWidget import SpectrumWidget


class ScanBrowserWidget(QWidget):
    def __init__(self, *args, **kwargs):
        QWidget.__init__(self, *args, **kwargs)
        self.mainlayout = QHBoxLayout(self)
        self.isAnnoOn = False

    def clearLayout(self, layout):
        for i in reversed(range(layout.count())):
            layout.itemAt(i).widget().setParent(None)

    def loadFile(self, file_path):
        self.isAnnoOn = False
        self.msexperimentWidget = QSplitter(Qt.Vertical)

        # data processing
        scans = self.readMS(file_path)

        # set Widgets
        self.spectrum_widget = SpectrumWidget()
        self.scan_widget = ScanTableWidget(scans)
        self.scan_widget.scanClicked.connect(self.redrawPlot)
        self.msexperimentWidget.addWidget(self.spectrum_widget)
        self.msexperimentWidget.addWidget(self.scan_widget)
        self.mainlayout.addWidget(self.msexperimentWidget)

        # default : first row selected.
        self.scan_widget.table_view.selectRow(0)

    def readMS(self, file_path):
        # Later: process other types of file
        exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(file_path, exp)
        return exp

    def redrawPlot(self):
        # set new spectrum and redraw
        self.spectrum_widget.setSpectrum(self.scan_widget.curr_spec)
        if self.isAnnoOn:  # update annotation list
            self.updateController()
        self.spectrum_widget.redrawPlot()

    def updateController(self):
        # for overrriding
        return



"""
--------------------------------------------------------------------------
                  OpenMS -- Open-Source Mass Spectrometry
--------------------------------------------------------------------------
Copyright The OpenMS Team -- Eberhard Karls University Tuebingen,
ETH Zurich, and Freie Universitaet Berlin 2002-2013.

This software is released under a three-clause BSD license:
 * Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
 * Neither the name of any author or any participating institution
   may be used to endorse or promote products derived from this software
   without specific prior written permission.
For a full list of authors, refer to the file AUTHORS.
--------------------------------------------------------------------------
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
import re

import pyopenms


def convertToRichMSSpectrum(input_):
    rs = pyopenms.RichMSSpectrum()
    for p in input_:
        rp = pyopenms.RichPeak1D()
        rp.setMZ(p.getMZ())
        rp.setIntensity(p.getIntensity())
        rs.push_back(rp)
    return rs


def convertToMSSpectrum(input_):
    spectrum = pyopenms.MSSpectrum()
    for p in input_:
        rp = pyopenms.Peak1D()
        rp.setMZ(p.getMZ())
        rp.setIntensity(p.getIntensity())
        spectrum.push_back(rp)
    return spectrum


"""
Two phospho scorers, the interface is available through the "score" function:

- Input:
  - PeptideHit (pyopenms.PeptideHit)
  - Spectrum (pyopenms.MSSpectrum)
  - [Scorer massDelta]

- Output:
    [ Score, NewSequence ]
"""


class PhosphoScorerAScore:
    def score(self, phit, spectrum, massDelta=0.5):
        nr_sites = phit.getSequence().toString().count("Phospho")
        rs = convertToRichMSSpectrum(spectrum)
        newhit = pyopenms.AScore().compute(phit, rs, massDelta, nr_sites)
        return [newhit.getScore(), newhit.getSequence()]


class PhosphoScorerSimple:
    def score(self, phit, spectrum):
        nr_sites = phit.getSequence().toString().count("Phospho")
        if nr_sites != 1:
            return [-1, pyopenms.AASequence()]
        return self.simplisticBinnedScoring(phit, spectrum)

    def simplisticBinnedScoring(self, phit, spectrum):
        """Simplistic phospho-scoring of a spectrum against a peptide hit.

        This function enumerates all possible locations for the
        phosphorylation and computes a similarity score between the
        experimental spectrum and the theoretical spectrum for each
        possibility.
        """
        seq = phit.getSequence().toString()
        seq = seq.replace("(Phospho)", "")
        possibilities = []
        spectrum_b = self.binSpectrum(spectrum)
        charge = 1
        # Iterate over all possible phosphosites
        for m in re.finditer("[STY]", seq):
            new_sequence = \
                seq[: m.start() + 1] + "(Phospho)" + seq[m.start() - 1:]
            new_aaseq = pyopenms.AASequence(new_sequence)
            # Generate theoretical spectrum
            spectrum_generator = pyopenms.TheoreticalSpectrumGenerator()
            rs = pyopenms.RichMSSpectrum()
            try:
                spectrum_generator.addPeaks(
                    rs, new_aaseq, pyopenms.Residue.ResidueType.YIon, charge
                )
                spectrum_generator.addPeaks(
                    rs, new_aaseq, pyopenms.Residue.ResidueType.BIon, charge
                )
            except AttributeError:
                # 1.11
                spectrum_generator.addPeaks(
                    rs, new_aaseq, pyopenms.ResidueType.YIon, charge
                )
                spectrum_generator.addPeaks(
                    rs, new_aaseq, pyopenms.ResidueType.BIon, charge
                )
            theor = convertToMSSpectrum(rs)
            theor_b = self.binSpectrum(theor)
            # Compare theoretical spectrum to experimental spectrum
            comp_score = self.compare_binnedSpectra(spectrum_b, theor_b)
            possibilities.append([comp_score, new_aaseq])

        # Sort the result by score, return the best scoring result
        possibilities.sort(lambda x, y: -cmp(x[0], y[0]))
        return possibilities[0]

    def compare_binnedSpectra(self, sp1, sp2):
        """Compare two binned spectra, return a similarity score

        The two binned spectra should be created by a call to binSpectrum."""

        start = max(min(sp1.keys()), min(sp2.keys()))
        end = min(max(sp1.keys()), max(sp2.keys())) + 1

        # Normalize input
        import math

        magnitude1 = math.sqrt(sum([v * v for v in sp1.values()]))
        magnitude2 = math.sqrt(sum([v * v for v in sp2.values()]))

        sp1_norm = dict([(k, v / magnitude1) for k, v in sp1.iteritems()])
        sp2_norm = dict([(k, v / magnitude2) for k, v in sp2.iteritems()])

        # Compute similarity score
        score = 0
        for i in range(start, end):
            score += sp1_norm[i] * sp2_norm[i]
        return score

    def binSpectrum(self, sp):
        """Bin a spectrum into single, 1 m/z wide bins"""
        assert sp.isSorted()
        peaks = sp.get_peaks()
        mz = sp.get_peaks()[:, 0]
        bins = {}
        start = int(min(mz))
        end = int(max(mz)) + 1
        for i in range(start, end):
            bins[i] = sum([p[1] for p in peaks if int(p[0]) == i])
        return bins


if __name__ == "__main__":
    print
    "This file is intended as library and not as Python executable,"
    "please do not execute it directly."




import pandas as pd


class TableDataFrame():

    def __init__(self, df: pd.DataFrame):
        self.df = df

    def getTable(self) -> pd.DataFrame:
        """
        Returns the current dataframe
        """
        return self.df

    def setTable(self, df: pd.DataFrame):
        """
        Sets the dataframe to the given
        """
        self.df = df

    def modifyGroup(self, rows: list, groupnum: int):
        """
        Let the user set the group for a list of selected rows.
        Needs a list of selected rows and the integer to which
        the group is set.
        """
        for row in rows:
            self.df.at[row, 'Fraction_Group'] = groupnum

    def modifyFraction(self, rows: list, *argv: int):
        """
        Let the user set the fraction for a list of selected rows.
        when only one number is given in argv one fraction is set
        to all selected entries, else it will count to from min to max
        and sets the group accordingly.
        Takes a list of selected rows and the number(s) how the fractions
        should be set.
        """
        if len(argv) == 1:
            for row in rows:
                self.df.at[row, 'Fraction'] = argv[0]
        elif len(argv) == 2:
            count = 0
            for row in rows:
                fracnum = argv[0] + count
                self.df.at[row, 'Fraction'] = fracnum
                if fracnum >= argv[1]:
                    count = 0
                else:
                    count += 1

    def modifyLabelSample(self, labelnum: int, continuous: bool):
        """
        Let the user change the multiplicity of the selected rows
        continuous should be boolean if true samplenumber counts
        through for all fraction groups otherwise it will start at
        1 for each fraction group
        Takes the number of labels and a boolean, which sets the option
        of continuing the samplenumber over groups.
        """
        # generate new dataframe from rows and copied rows of labels
        ndf = []
        for row in self.df.index:
            for cl in range(labelnum):
                label = cl+1
                self.df.at[row, 'Label'] = label
                dfrow = self.df.loc[[row]]
                ndf.append(dfrow)
        ndf = pd.concat(ndf, ignore_index=True)

        # associate the samplenum to the label
        if continuous:
            prevsample = 0
            for row in ndf.index:
                if (row == 0):
                    samplenum = 1
                    ndf.at[row, 'Sample'] = samplenum
                elif(ndf.at[row, 'Fraction_Group'] !=
                     ndf.at[row-1, 'Fraction_Group']):
                    prevsample = ndf.at[row-1, 'Sample']
                    samplenum = ndf.at[row, 'Label'] + prevsample
                    ndf.at[row, 'Sample'] = samplenum
                else:
                    samplenum = ndf.at[row, 'Label'] + prevsample
                    ndf.at[row, 'Sample'] = samplenum
        else:
            for row in ndf.index:
                samplenum = ndf.at[row, 'Label']
                ndf.at[row, 'Sample'] = samplenum

        self.df = ndf

    def rmvRow(self, rows: list):
        """
        Let the user remove a list of rows.
        Takes list of selected rows.
        """
        self.df.drop(rows, inplace=True)
        self.df.reset_index(drop=True, inplace=True)

    def modifyField(self, row: int, column: int, newvalue: int):
        col = self.df.columns[column]
        self.df.at[row, col] = newvalue



import os
import sys
from collections import namedtuple

import numpy as np
import pandas as pd
import pyqtgraph as pg
from PyQt5.QtCore import Qt
from PyQt5.QtGui import (
    QStandardItemModel,
    QStandardItem,
    QPainter,
    QIcon,
    QBrush,
    QColor,
    QPen,
    QPixmap,
    QIntValidator,
)
from PyQt5.QtWidgets import (
    QApplication,
    QMainWindow,
    QVBoxLayout,
    QHBoxLayout,
    QWidget,
    QMessageBox,
    QPushButton,
    QLabel,
    QAction,
    QFileDialog,
    QTableView,
    QSplitter,
    QDialog,
    QToolButton,
    QLineEdit,
    QRadioButton,
    QGroupBox,
    QFormLayout,
    QDialogButtonBox,
)
from matplotlib import cm
from pyqtgraph import PlotWidget

sys.path.insert(0, "../view")
from SpecViewer import ScanBrowserWidget, App

# import pyopenms.Constants
# define Constant locally until bug in pyOpenMS is fixed
PROTON_MASS_U = 1.0072764667710
C13C12_MASSDIFF_U = 1.0033548378

# structure for each input masses
MassDataStruct = namedtuple(
    "MassDataStruct",
    "mz_theo_arr \
                            startRT endRT maxIntensity scanCount \
                            color marker",
)
#                            isMono isAvg
PeakAnnoStruct = namedtuple(
    "PeakAnnoStruct",
    "mz intensity text_label \
                            symbol symbol_color",
)
LadderAnnoStruct = namedtuple(
    "LadderAnnoStruct",
    "mz_list \
                            text_label_list color",
)
TOL = 1e-5  # ppm

SymbolSet = ("o", "s", "t", "t1", "t2", "t3", "d", "p", "star")
RGBs = [
    [0, 0, 200],
    [0, 128, 0],
    [19, 234, 201],
    [195, 46, 212],
    [237, 177, 32],
    [54, 55, 55],
    [0, 114, 189],
    [217, 83, 25],
    [126, 47, 142],
    [119, 172, 48],
]
Symbols = pg.graphicsItems.ScatterPlotItem.Symbols


class MassList:
    def __init__(self, file_path):
        if not file_path:
            self.mass_list = []
            return

        self.data = pd.read_csv(file_path, sep="\t", header=0)
        self.isFDresult = self.isValidFLASHDeconvFile()
        self.setRTMassDict()
        self.setMassList(self.data)

    def setMassList(self, df):
        if self.isFDresult:  # parsing a result file from FLASHDeconv
            self.mass_list = df["MonoisotopicMass"].to_numpy().ravel().tolist()
        else:
            self.mass_list = df.to_numpy().ravel().tolist()

    def setMassStruct(self, cs_range=[2, 100]):
        mds_dict = {}

        for mNum, mass in enumerate(self.mass_list):
            mds_dict[mass] = self.setMassDataStructItem(mNum, mass, cs_range)
        return mds_dict

    def getMassStruct(self, masslist, cs_range=[2, 100]):
        mds_dict = {}

        for mass in masslist:
            mNum = self.mass_list.index(mass)
            mds_dict[mass] = self.setMassDataStructItem(mNum, mass, cs_range)
        return mds_dict

    def setMassDataStructItem(self, index, mass, cs_range):
        marker = SymbolSet[index % len(SymbolSet)]
        color = RGBs[index % len(RGBs)]
        theo_mz = self.calculateTheoMzList(mass, cs_range)
        rt_s = 0
        rt_e = sys.maxsize
        mi = 0
        c = 0
        try:
            if mass in self.RTMassDict.keys():
                rt_s = float(self.RTMassDict[mass]["StartRetentionTime"])
                rt_e = float(self.RTMassDict[mass]["EndRetentionTime"])
                mi = float(self.RTMassDict[mass]["MaxIntensity"])
                c = int(self.RTMassDict[mass]["MassCount"])
        except (AttributeError, NameError):  # no input mass file
            pass

        return MassDataStruct(
            mz_theo_arr=theo_mz,
            startRT=rt_s,
            endRT=rt_e,
            maxIntensity=mi,
            scanCount=c,
            marker=marker,
            color=color,
        )

    def calculateTheoMzList(self, mass, cs_range, mz_range=(0, 0)):
        theo_mz_list = []
        for cs in range(cs_range[0], cs_range[1] + 1):
            mz = (mass + cs * PROTON_MASS_U) / cs
            iso = [
                C13C12_MASSDIFF_U / cs * i + mz for i in range(10)
            ]  # 10 should be changed based on the mass, later.
            theo_mz_list.append((cs, np.array(iso)))
        return theo_mz_list

    def addNewMass(self, new_mass, index, cs_range):
        return self.setMassDataStructItem(index, new_mass, cs_range)

    def isValidFLASHDeconvFile(self):
        col_needed = [
            "MonoisotopicMass",
            "AverageMass",
            "StartRetentionTime",
            "EndRetentionTime",
        ]
        result = all(elem in list(self.data) for elem in col_needed)
        if result:
            return True
        return False

    def setRTMassDict(self):
        self.RTMassDict = dict()
        if self.isFDresult:
            self.RTMassDict = self.data.set_index(
                "MonoisotopicMass").to_dict("index")


class FeatureMapPlotWidget(PlotWidget):
    def __init__(self, mass_data, parent=None, dpi=100):
        PlotWidget.__init__(self)
        self.data = mass_data
        self.setLabel("bottom", "Retension Time (sec)")
        self.setLabel("left", "Mass (Da)")
        self.setLimits(yMin=0, xMin=0)
        self.showGrid(True, True)
        # self.setBackground('k')
        self.drawPlot()
        # self.setColorbar()

    # def setColorbar(self):
    #     self.img = pg.ImageItem()
    #     self.getPlotItem().addItem(self.img)
    #     self.img.setLookupTable(self.pg_cmap.getLookupTable)
    #     self.hist = pg.HistogramLUTItem()
    #     self.hist.setImageItem(self.img)
    #     # self.addItem(self.hist)

    def drawPlot(self):
        cmap = self.getColorMap()

        for mass, mds in self.data.items():
            spi = pg.ScatterPlotItem(
                size=10,  # pen=pg.mkPen(None),
                brush=pg.mkBrush(cmap.mapToQColor(mds.maxIntensity)),
            )
            self.addItem(spi)
            spots = [{"pos": [i, mass]}
                     for i in np.arange(mds.startRT, mds.endRT, 1)]
            # print([i for i in np.arange(mds.startRT, mds.endRT, 1)])
            spi.addPoints(spots)

    def getColorMap(self):
        miList = self.getMassIntensityDict()
        colormap = cm.get_cmap("plasma")
        colormap._init()
        lut = \
            (colormap._lut * 255).view(np.ndarray)[: colormap.N]
        # Convert matplotlib colormap from 0-1 to 0 -255 for Qt
        self.pg_cmap = pg.ColorMap(pos=miList, color=lut)
        return self.pg_cmap

    def getMassIntensityDict(self):
        # miDict = dict()
        miList = list()
        for m, mds in self.data.items():
            # miDict[m] = mds.maxIntensity
            miList.append(mds.maxIntensity)
        return miList


class PlotWindow(QMainWindow):
    def __init__(self, mlc, mass_data, parent=None):
        QMainWindow.__init__(self, parent)
        self.setWindowTitle("Feature map")
        cWidget = QWidget()
        self.setCentralWidget(cWidget)
        self.layout = QHBoxLayout(cWidget)

        fmWidget = FeatureMapPlotWidget(mass_data)
        self.colormap = fmWidget.pg_cmap

        self.layout.addWidget(fmWidget)
        # cWidget.addItem(fmWidget.hist)

        # self.addColorBar(mlc)

    def addColorBar(self, mlc):
        self.img = pg.ColorMapWidget()
        self.img.setFields([("MaxIntensity", {})])
        self.img.map(mlc.data)
        # self.img.setLookupTable(self.colormap.getLookupTable)
        # https://groups.google.com/forum/#!searchin/pyqtgraph/color$20scale$20plotwidget/pyqtgraph/N4ysAIhPBgo/JO36xjz1BwAJ
        # imgV = pg.ImageView(view=pg.PlotItem())
        self.layout.addWidget(self.img)


class ControllerWidget(QWidget):
    def __init__(self, mass_path, plot, *args):
        QWidget.__init__(self, *args)
        self.mass_path = mass_path
        hbox = QVBoxLayout()
        self.setMaximumWidth(350)
        self.spectrum_widget = plot

        # data processing
        self.mlc = MassList(mass_path)
        self.total_masses = self.mlc.setMassStruct()
        self.masses = dict()  # initialization

        self.setFeatureMapButton()
        self.setMassTableView()
        self.setMassLineEdit()
        self.setParameterBox()

        hbox.addWidget(self.fmButton)
        hbox.addWidget(self.massTable)
        hbox.addLayout(self.massLineEditLayout)
        hbox.addWidget(self.paramBox)
        # hbox.addWidget(self.paramButton)
        self.setLayout(hbox)

    def _updatePlot(self):
        # uncheck all checkboxes
        for index in range(self.model.rowCount()):
            item = self.model.item(index)
            if item.isCheckable():
                item.setCheckState(Qt.Unchecked)

        # reset parameter default value
        self.csMinLineEdit.setText("2")
        self.csMaxLineEdit.setText("100")

        self._data_visible = []
        self.spectrum_widget.setPeakAnnotations(self.getPeakAnnoStruct())
        self.spectrum_widget.setLadderAnnotations(self.getLadderAnnoStruct())

        self.spectrum_widget.redrawPlot()

    def setFeatureMapButton(self):
        self.fmButton = QPushButton()
        self.fmButton.setText("Draw feature map")
        self.fmButton.clicked.connect(self.loadFeatureMapPlot)

    def setMassTableView(self):
        # set controller widgets
        self.massTable = QTableView()
        self.model = QStandardItemModel(self)
        self.model.setHorizontalHeaderLabels(["Masses"])
        self.model.itemChanged.connect(self.check_check_state)
        self.massTable.setModel(self.model)
        self.massTable.setColumnWidth(0, 300)
        self._data_visible = []

    def setMassLineEdit(self):
        self.massLineEditLayout = QHBoxLayout()
        self.massLineEdit = QLineEdit()
        self.massLineEditButton = QToolButton()
        self.massLineEditButton.setText("Add")
        self.massLineEditLabel = QLabel("Add mass to list")
        self.massLineEditLabel.setBuddy(self.massLineEdit)
        self.massLineEditLayout.addWidget(self.massLineEditLabel)
        self.massLineEditLayout.addWidget(self.massLineEdit)
        self.massLineEditLayout.addWidget(self.massLineEditButton)
        self.massLineEditButton.clicked.connect(self.addMassToListView)

    def setParameterBox(self):
        self.paramBox = QGroupBox("Parameter")
        paramLayout = QVBoxLayout(self.paramBox)

        paramLayout.addLayout(self.setChargeRangeLineEdit())

        self.paramButton = QPushButton()
        self.paramButton.setText("Reload")
        self.paramButton.clicked.connect(self.redrawAnnotationsWithParam)
        paramLayout.addWidget(self.paramButton)

    def setChargeRangeLineEdit(self):
        self.cs_range = [2, 100]

        csRangeEditLayout = QHBoxLayout()
        self.csMinLineEdit = QLineEdit()
        self.csMaxLineEdit = QLineEdit()
        csMinLineEditLabel = QLabel("min")
        csMaxLineEditLabel = QLabel("max")
        self.csMinLineEdit.setText("2")
        self.csMaxLineEdit.setText("100")
        csMinLineEditLabel.setBuddy(self.csMinLineEdit)
        csMaxLineEditLabel.setBuddy(self.csMaxLineEdit)

        csRangeEditLabel = QLabel("Charge range:")
        csRangeEditLabel.setToolTip(
            "Minimum Charge should be equal to or larger than 2"
        )
        csRangeEditLayout.addWidget(csRangeEditLabel)
        csRangeEditLayout.addWidget(csMinLineEditLabel)
        csRangeEditLayout.addWidget(self.csMinLineEdit)
        csRangeEditLayout.addWidget(csMaxLineEditLabel)
        csRangeEditLayout.addWidget(self.csMaxLineEdit)

        return csRangeEditLayout

    def setMassListExportButton(self):
        self.setmassbutton = ""

    def loadFeatureMapPlot(self):
        if not self.mlc.isFDresult:
            self.errorDlg = QMessageBox()
            self.errorDlg.setIcon(QMessageBox.Critical)
            self.errorDlg.setWindowTitle("ERROR")
            self.errorDlg.setText(
                "Input mass file is not formatted as FLASHDeconv result file."
            )
            self.errorDlg.exec_()
            return
        self.fm_window = PlotWindow(self.mlc, self.total_masses)
        self.fm_window.show()

    def updateMassTableView(self, scan_rt):

        self.masses = self.getMassStructWithRT(scan_rt)
        self.model.removeRows(0, self.model.rowCount())
        self.model.setHorizontalHeaderLabels(["Masses"])
        for mass, mStruct in self.masses.items():
            self.setListViewWithMass(mass, mStruct)
        self.massTable.setColumnWidth(0, 300)
        # update annotation lists
        self._data_visible = []
        self.spectrum_widget.setPeakAnnotations(self.getPeakAnnoStruct())
        self.spectrum_widget.setLadderAnnotations(self.getLadderAnnoStruct())

    def getMassStructWithRT(self, scan_rt):
        new_dict = dict()
        for mass, mds in self.total_masses.items():
            if (mds.startRT == 0 and mds.endRT == sys.maxsize) or (
                    scan_rt >= mds.startRT and scan_rt <= mds.endRT
            ):
                new_dict[mass] = mds
        return new_dict

    def redrawAnnotationsWithParam(self):
        minCs = self.csMinLineEdit.text()
        maxCs = self.csMaxLineEdit.text()

        if self.isError_redrawAnnotationsWithParam(minCs, maxCs):
            self.csMinLineEdit.setText("2")
            self.csMaxLineEdit.setText("100")
            return

        # redraw
        self.cs_range = [int(minCs), int(maxCs)]
        self.masses = self.mlc.getMassStruct(self.masses, self.cs_range)
        self._updatePlot()

    def isError_redrawAnnotationsWithParam(self, minCs, maxCs):
        v = QIntValidator()
        v.setBottom(2)
        self.csMinLineEdit.setValidator(v)
        self.csMaxLineEdit.setValidator(v)

        if int(minCs) > int(maxCs):
            return True

        return False

    def getSymbolIcon(self, symbol, color):

        px = QPixmap(20, 20)
        px.fill(Qt.transparent)
        qp = QPainter(px)
        qpen = QPen(Qt.black, 0.05)
        qc = QColor()
        qc.setRgb(color[0], color[1], color[2])

        qp.setRenderHint(QPainter.Antialiasing)
        qp.setPen(qpen)
        qp.setBrush(QBrush(qc))
        qp.translate(10, 10)
        qp.scale(20, 20)
        qp.drawPath(Symbols[symbol])
        qp.end()

        return QIcon(px)

    def getPeakAnnoStruct(self):
        pStructList = []
        for mass, mass_strc in self.masses.items():
            theo_list = mass_strc.mz_theo_arr

            for theo in theo_list:  # theo : [0] cs [1] iso mz list
                exp_p = self.findNearestPeakWithTheoPos(
                    theo[1][0])  # Monoisotopic only
                if exp_p is None:
                    continue
                pStructList.append(
                    PeakAnnoStruct(
                        mz=exp_p.getMZ(),
                        intensity=exp_p.getIntensity(),
                        text_label="+" + str(theo[0]),
                        symbol=mass_strc.marker,
                        symbol_color=mass_strc.color,
                    )
                )

        return pStructList

    def getLadderAnnoStruct(self):
        lStructDict = dict()
        xlimit = [self.spectrum_widget.minMZ, self.spectrum_widget.maxMZ]

        for mass, mass_strc in self.masses.items():
            mass = str(mass)
            if mass in self._data_visible:
                # calculating charge ladder
                theo_list = mass_strc.mz_theo_arr
                t_mz_list = []
                txt_list = []

                for theo in theo_list:  # theo : [0] cs [1] iso mz list
                    # plotting only theoretical mz valule
                    # within experimental mz range
                    if (theo[1][0] <= xlimit[0]) | (theo[1][-1] >= xlimit[1]):
                        continue

                    for index, mz in enumerate(theo[1]):
                        t_mz_list.append(mz)
                        txt_list.append("+%d[%d]" % (theo[0], index))
                lStructDict[mass] = LadderAnnoStruct(
                    mz_list=np.array(t_mz_list),
                    text_label_list=np.array(txt_list),
                    color=mass_strc.color,
                )
            else:
                self.spectrum_widget.clearLadderAnnotation(mass)
        return lStructDict

    def findNearestPeakWithTheoPos(self, theo_mz, tol=-1):
        nearest_p = self.spectrum_widget.spec[
            self.spectrum_widget.spec.findNearest(theo_mz)
        ]
        if tol == -1:
            tol = TOL * theo_mz  # ppm
        if abs(theo_mz - nearest_p.getMZ()) > tol:
            return None
        if nearest_p.getIntensity() == 0:
            return None
        return nearest_p

    def setListViewWithMass(self, mass, mStruct):

        icon = self.getSymbolIcon(mStruct.marker, mStruct.color)
        item = QStandardItem(icon, str(mass))
        item.setCheckable(True)

        self.model.appendRow([item])

    def check_check_state(self, i):
        if not i.isCheckable():  # Skip data columns.
            return

        mass = i.text()
        checked = i.checkState() == Qt.Checked

        if mass in self._data_visible:
            if not checked:
                self._data_visible.remove(str(mass))
                self.spectrum_widget.setLadderAnnotations(
                    self.getLadderAnnoStruct())
                self.spectrum_widget.redrawLadderAnnotations()
        else:
            if checked:
                self._data_visible.append(str(mass))
                self.spectrum_widget.setLadderAnnotations(
                    self.getLadderAnnoStruct())
                self.spectrum_widget.redrawLadderAnnotations()

    def addMassToListView(self):
        new_mass = self.massLineEdit.text()
        self.massLineEdit.clear()
        try:
            new_mass = float(new_mass)
        except Exception:
            return
        new_mass_str = self.mlc.addNewMass(
            new_mass, len(self.masses), self.cs_range)
        self.masses[new_mass] = new_mass_str

        # redraw
        self._updatePlot()
        self.setListViewWithMass(new_mass, new_mass_str)


class ScanBrowserWidget_FDV(ScanBrowserWidget):
    def updateController(self):  # overriding from ScanBrowserWidget
        self.controller.updateMassTableView(self.scan_widget.curr_spec.getRT())

    def annotation_FLASHDeconv(self, mass_path):
        self.controller = ControllerWidget(mass_path, self.spectrum_widget)
        self.isAnnoOn = True
        # annotate first scan
        self.redrawPlot()

        # Adding Splitter
        self.splitter_spec_contr = QSplitter(Qt.Horizontal)
        self.splitter_spec_contr.addWidget(self.msexperimentWidget)
        self.splitter_spec_contr.addWidget(self.controller)
        self.mainlayout.addWidget(self.splitter_spec_contr)


class FDInputDialog(QDialog):
    def __init__(self):
        super(FDInputDialog, self).__init__()

        self.setWindowTitle("Starting FLASHDeconv Visualization")
        self.setupUI()

    def setupUI(self):
        mainlayout = QVBoxLayout()
        self.setLayout(mainlayout)
        self.setErrorMessageBox()

        # widgets
        self.mzmlFileLineEdit = QLineEdit()
        self.mzmlFileButton = QToolButton()
        self.mzmlFileButton.setText("...")
        self.massFileButton = QToolButton()
        self.massFileLineEdit = QLineEdit()
        self.massFileButton.setText("...")
        self.mTypeButton1 = QRadioButton("Monoisotopic")
        self.mTypeButton2 = QRadioButton("Average")
        self.mTypeButton1.setChecked(True)
        self.tolerance = QLineEdit()
        self.tolerance.setText("10")

        self.setFormGroupBox()

        # connection
        self.mzmlFileButton.clicked.connect(self.openMzMLFileDialog)
        self.massFileButton.clicked.connect(self.openMassFileDialog)

        # from group box - ok/cancel
        buttonBox = QDialogButtonBox(
            QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        buttonBox.accepted.connect(self.handleException)
        buttonBox.rejected.connect(self.reject)

        # layout
        mainlayout.addWidget(self.formGroupBox)
        mainlayout.addWidget(buttonBox)

    def setFormGroupBox(self):
        self.formGroupBox = QGroupBox()
        layout = QFormLayout()

        mzmlLyt = QHBoxLayout()
        mzmlLyt.addWidget(self.mzmlFileLineEdit)
        mzmlLyt.addWidget(self.mzmlFileButton)
        massLyt = QHBoxLayout()
        massLyt.addWidget(self.massFileLineEdit)
        massLyt.addWidget(self.massFileButton)
        mTypeLyt = QHBoxLayout()
        mTypeLyt.addWidget(self.mTypeButton1)
        mTypeLyt.addWidget(self.mTypeButton2)
        tolLyt = QHBoxLayout()
        tolLyt.addWidget(self.tolerance)

        layout.addRow(QLabel("Input mzML File:"), mzmlLyt)
        layout.addRow(QLabel("Input Mass File:"), massLyt)
        layout.addRow(QLabel("Input Mass Type:"), mTypeLyt)
        layout.addRow(QLabel("Tolerance(ppm):"), tolLyt)
        self.formGroupBox.setLayout(layout)

    def openMzMLFileDialog(self):
        fileName, _ = QFileDialog.getOpenFileName(
            self, "Open File ", "", "mzML Files (*.mzML)"
        )
        if fileName:
            self.mzmlFileLineEdit.setText(fileName)

    def openMassFileDialog(self):
        fileName, _ = QFileDialog.getOpenFileName(
            self, "Open File ", "", "Text Files (*.csv, *tsv)"
        )
        if fileName:
            self.massFileLineEdit.setText(fileName)

    def setErrorMessageBox(self):
        self.errorDlg = QMessageBox()
        self.errorDlg.setIcon(QMessageBox.Critical)
        self.errorDlg.setWindowTitle("ERROR")

    def handleException(self):

        if not os.path.exists(self.mzmlFileLineEdit.text()):
            self.errorDlg.setText("Input mzML file doesn't exist.")
            self.errorDlg.exec_()
            return

        if self.massFileLineEdit.text() and not os.path.exists(
                self.massFileLineEdit.text()
        ):
            self.errorDlg.setText("Input mass file doesn't exist.")
            self.errorDlg.exec_()
            return

        try:
            float(self.tolerance.text())
        except Exception:
            self.errorDlg.setText("Tolerance is not a number.")
            self.errorDlg.exec_()
            return

        self.accept()


class App_FDV(App):
    def setOpenMSWidget(self):  # overriding from App
        if self.windowLay.count() > 0:
            self.clearLayout(self.windowLay)
        self.scanbrowser = ScanBrowserWidget_FDV(self)
        self.windowLay.addWidget(self.scanbrowser)

    def setToolMenu(self):  # overriding from App
        # FLASHDeconv Viewer
        fdAct = QAction("FLASHDeconvV", self)
        fdAct.triggered.connect(self.startFLASHDeconvV)
        self.toolMenu.addAction(fdAct)

    def startFLASHDeconvV(self):

        inputDlg = FDInputDialog()
        if inputDlg.exec_():  # data accepted
            self.mzmlPath = inputDlg.mzmlFileLineEdit.text()
            self.massPath = inputDlg.massFileLineEdit.text()
            self.tol = inputDlg.tolerance.text()
            self.isAvg = inputDlg.mTypeButton2.isChecked()

            if self.isAvg:
                print("Calculate with AVG mass")

            self.setOpenMSWidget()
            self.scanbrowser.loadFile(self.mzmlPath)
            self.scanbrowser.annotation_FLASHDeconv(self.massPath)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    ex = App_FDV()
    ex.show()
    sys.exit(app.exec_())



import sys, os
from PyQt5.QtWidgets import QApplication, QVBoxLayout, QMainWindow, \
     QDesktopWidget, QWidget, QPushButton, QAction
from PyQt5.QtCore import Qt
sys.path.append(os.getcwd()+'/../view')
from ConfigView import ConfigView


class XMLViewer(QMainWindow):
    """
    Widget for visualizing configuration data
    """

    def __init__(self):
        QMainWindow.__init__(self)
        self.initUI()

    def initUI(self):
        '''
        sets the window with all applications and widgets
        which are loaded from the ConfigView.py file
        '''
        cview = ConfigView()

        menubar = self.menuBar()
        menubar.setNativeMenuBar(False)
        fileMenu = menubar.addMenu('&File')
        saveAction = QAction("&Save File", self)
        loadAction = QAction("&Load File", self)

        saveAction.setDisabled(True)

        fileMenu.addAction(loadAction)
        fileMenu.addAction(saveAction)

        loadAction.triggered.connect(cview.openXML)
        saveAction.triggered.connect(cview.saveFile)

        self.setCentralWidget(cview)
        self.resize(800, 1000)
        self.center()
        self.setWindowTitle('ini File Viewer')
        self.show()

    def center(self):
        """
        centers the widget to the screen
        """
        qr = self.frameGeometry()
        cp = QDesktopWidget().availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

"""
if __name__ == '__main__':

    app = QApplication(sys.argv)
    ex = XMLViewer()
    sys.exit(app.exec_())
"""



""" 
mzTabTableWidget
----------------
This script allows the user to transfer information about proteins and psms from a mzTab file into two tables, 
one containing the proteins, the other one containing the psms.
By clicking on a row, the tables get updated regarding their listed proteins or psms.
Once you choose a protein/psm, the table displays only those psms/proteins that are linked to one another.
This tool is designed to accept mzTab files. It is required to save those files under '.../examples/data/' or 
change the path within the InitWindow.
"""
import sys
import webbrowser
from PyQt5 import QtGui, QtWidgets
from PyQt5.QtWidgets import QApplication, QWidget, QTableWidget, QVBoxLayout, QTableWidgetItem, QPushButton, QFileDialog


class mzTabTableWidget(QWidget):
    """
        class to create mzTabTableWidget to display two tables containing data of mzTab file
        ...
        Attributes
        ----------
        title : string
            title of widget

        top, left, width, height : int
            attributes regarding size of widget

        fileLoaded : boolean
            boolean that describes if file is loaded

        PSMFull : list of lists
            list containing all rows of full psm table

        PRTFull : list of lists
            list containing all rows of fill prt table

        PRTFiltered: list of lists
            list containing all rows of filtered prt table

        PSMFiltered: list of lists
            list containing all rows of filtered psm table

        PRTColumn : list of booleans
            list of booleans that describe whether prt column is shown or hidden

        PSMColumn : list of booleans
            list of booleans that describe whether psm column is shown or hidden

        tablePRTFull : QTableWidget
            table widget that includes full prt table

        tablePSMFull : QTableWidget
            table widget that includes full psm table

        tablePRTFiltered : QTableWidget
            table widget that includes filtered prt table

        tablePSMFiltered: QTableWidget
            table widget that includes filtered psm table

        vBoxPRT : QVBoxLayout
            layout for PRT table

        vBoxPSM : QVBoxLayout
            layout for PSM table

        outerVBox : QVBoxLayout
            main layout for widget

        Methods
        -------
        __init__(self)
            defines window size and title and attributes for PRT and PSM tables

        InitWindow(self)
            creates window, manages click events and adds widgets to layout

        readFile(self, file)
            loads new file into tables

        parser(self, file)
            parses mzTab file and saves PRT and PSM information

        initTables(self)
            draws prt and psm tables with headers

        createTable(self, table, content)
            fills tables with content

        hidePRTColumns(self)
            hides constant prt columns

        hidePSMColumns(self)
            hides constant psm columns

        PRTClicked(self, item)
            sets relevantContent of prt table and decides whether to use filter

        PSMClicked(self, item)
            sets relevantContent of psm table and decides whether to use filter

        filterPRT(self, accession)
            filters prt table

        filterPSM(self, accession)
            filters psm table

        unfilterPRT(self)
            deactivates filter of prt table

        unfilterPSM(self)
            deactivates filter of psm table

        browsePRT(sef, item)
            opens uniport website when prt table item is double clicked

        browsePSM(self, item)
            opens uniport website when psm table item is double clicked
            
        """
    def __init__(self):

        super().__init__()

        self.title = "mzTabTableWidget"
        self.top = 100
        self.left = 100
        self.width = 500
        self.height = 500
        self.tableRows = 5

        self.fileLoaded = False

        self.PRTFull = []
        self.PSMFull = []

        self.PRTFiltered = []
        self.PSMFiltered = []

        self.PRTColumn = [True]
        self.PSMColumn = [True]

        self.selectedPRT = ""
        self.selectedPSM = ""

        self.tablePRTFull = QTableWidget()
        self.tablePSMFull = QTableWidget()

        self.tablePRTFiltered = QTableWidget()
        self.tablePSMFiltered = QTableWidget()

        self.vBoxPRT = QVBoxLayout()
        self.vBoxPSM = QVBoxLayout()

        self.outerVBox = QVBoxLayout()

        self.InitWindow()

    def InitWindow(self):
        """
        class to initiate window and its layout and define table parameters

        """
        self.setWindowIcon(QtGui.QIcon("icon.png"))
        self.setWindowTitle(self.title)
        self.setGeometry(self.top, self.left, self.width, self.height)

        # hide tables for now
        self.tablePRTFull.setHidden(True)
        self.tablePSMFull.setHidden(True)
        self.tablePRTFiltered.setHidden(True)
        self.tablePSMFiltered.setHidden(True)

        # connect click events to functions that update tables
        self.tablePRTFull.itemClicked.connect(self.PRTClicked)
        self.tablePRTFiltered.itemClicked.connect(self.PRTClicked)
        self.tablePSMFull.itemClicked.connect(self.PSMClicked)
        self.tablePSMFiltered.itemClicked.connect(self.PSMClicked)

        self.tablePRTFull.itemDoubleClicked.connect(self.browsePRT)
        self.tablePRTFiltered.itemDoubleClicked.connect(self.browsePRT)
        self.tablePSMFull.itemDoubleClicked.connect(self.browsePSM)
        self.tablePSMFiltered.itemDoubleClicked.connect(self.browsePSM)

        #add tables to layout
        self.vBoxPRT.addWidget(self.tablePRTFull)
        self.vBoxPRT.addWidget(self.tablePRTFiltered)

        self.vBoxPSM.addWidget(self.tablePSMFull)
        self.vBoxPSM.addWidget(self.tablePSMFiltered)

        self.outerVBox.addLayout(self.vBoxPRT)
        self.outerVBox.addLayout(self.vBoxPSM)

        self.setLayout(self.outerVBox)
        self.show()

    def readFile(self, file):
        """
        class to load new file into tables
        
        """
        if self.fileLoaded:
            # clear table content for now
            self.tablePRTFull.clear()
            self.tablePSMFull.clear()
            self.tablePSMFiltered.clear()
            self.tablePRTFiltered.clear()

            self.tablePRTFull.setRowCount(0)
            self.tablePSMFull.setRowCount(0)
            self.tablePSMFiltered.setRowCount(0)
            self.tablePRTFiltered.setRowCount(0)

            self.PRTFull.clear()
            self.PSMFull.clear()

            self.PRTFiltered.clear()
            self.PSMFiltered.clear()

            self.PRTColumn.clear()
            self.PSMColumn.clear()

            self.PRTColumn = [True]
            self.PSMColumn = [True]

        # parse file
        self.parser(file)

        # save column length in PRTColumn and PSMColumn
        self.PRTColumn *= len(self.PRTFull[1])
        self.PSMColumn *= len(self.PSMFull[1])

        # create full tables
        self.initTables()
        self.createTable(self.tablePRTFull, self.PRTFull)
        self.createTable(self.tablePSMFull, self.PSMFull)

        # hide columns that are empty or have the same content in each line
        self.hidePRTColumns()
        self.hidePSMColumns()

        # show full tables for now
        self.tablePRTFull.setHidden(False)
        self.tablePSMFull.setHidden(False)

        self.fileLoaded = True


    def parser(self, file):
        """parses the given mzTab file and saves PRT and PSM information
        Parameters
        ----------
        file : str
            The file path of the mzTab file
        """

        with open(file) as inp:
            for line in inp:
                if line.startswith("PRH"):
                    self.PRTFull.append(line.strip().split('\t'))
                elif line.startswith("PRT") and not line.endswith("protein_details\n"):
                    self.PRTFull.append(line.strip().split('\t'))
                elif line.startswith("PSH") or line.startswith("PSM"):
                    self.PSMFull.append(line.strip().split('\t'))

        # delete first row of each table
        for item in self.PRTFull:
            item.pop(0)

        for item in self.PSMFull:
            item.pop(0)



    def initTables(self):
        """draws protein and psm tables with headers"""

        self.tablePRTFull.setRowCount(len(self.PRTFull))
        self.tablePRTFull.setColumnCount(len(self.PRTFull[0]))
        self.tablePRTFull.setHorizontalHeaderLabels(self.PRTFull[0])

        self.tablePSMFull.setRowCount(len(self.PSMFull))
        self.tablePSMFull.setColumnCount(len(self.PSMFull[0]))
        self.tablePSMFull.setHorizontalHeaderLabels(self.PSMFull[0])

        self.tablePRTFiltered.setRowCount(0)
        self.tablePRTFiltered.setColumnCount(len(self.PRTFull[0]))
        self.tablePRTFiltered.setHorizontalHeaderLabels(self.PRTFull[0])

        self.tablePSMFiltered.setRowCount(0)
        self.tablePSMFiltered.setColumnCount(len(self.PSMFull[0]))
        self.tablePSMFiltered.setHorizontalHeaderLabels(self.PSMFull[0])

        """removes now unnecessary header information from content lists """
        self.PRTFull.pop(0)
        self.PSMFull.pop(0)

    def createTable(self, table, content):
        """parameters: tableWidget to draw content in. Content to be drawn in list form"""
        """Setting count to zero empties the table. Then table is (re-)filled with specified content"""
        table.setRowCount(0)
        table.setRowCount(len(content))

        j = 0
        k = 0

        for item in content[0:]:
            while k < (len(content)):
                while j < (len(item)):
                    table.setItem(k, j, QTableWidgetItem(item[j]))
                    j += 1
                else:
                    k += 1
                    j = 0
                break

        self.tablePRTFull.resizeColumnsToContents()  # resize columns
        self.tablePSMFull.resizeColumnsToContents()  # resize columns
        self.tablePRTFiltered.resizeColumnsToContents()  # resize columns
        self.tablePSMFiltered.resizeColumnsToContents()  # resize columns

    def hidePRTColumns(self):
        """hides constant columns in PRT table by default by checking if every value equals"""
        i = 0
        j = 0
        k = 0

        while i < len(self.PRTFull) - 1:
            while j < len(self.PRTFull[i]):
                if self.PRTColumn[j]:
                    if self.PRTFull[i][j] != self.PRTFull[i + 1][j]:
                        self.PRTColumn[j] = False
                j += 1
            i += 1

        while k < len(self.PRTColumn):
            if self.PRTColumn[k]:
                self.tablePRTFull.setColumnHidden(k, True)
                self.tablePRTFiltered.setColumnHidden(k, True)
            k += 1

    def hidePSMColumns(self):
        """hides constant columns in PSM table by default by checking if every value equals"""
        i = 0
        j = 0
        k = 0

        while i < len(self.PSMFull) - 1:
            while j < len(self.PSMFull[i]):
                if self.PSMColumn[j]:
                    if self.PSMFull[i][j] != self.PSMFull[i + 1][j]:
                        self.PSMColumn[j] = False
                j += 1
            i += 1

        while k < len(self.PSMColumn):
            if self.PSMColumn[k]:
                self.tablePSMFull.setColumnHidden(k, True)
                self.tablePSMFiltered.setColumnHidden(k, True)
            k += 1

    def PRTClicked(self, item):
        """ sets relevantContent of prt table and decides whether to use filter or not
            ...
            Parameters
            ----------
            item : QTableWidgetItem
                clicked table widget item
        """
        if self.tablePRTFull.isHidden():
            relevantContent = self.PRTFiltered
        else:
            relevantContent = self.PRTFull

        accession = relevantContent[item.row()][0]

        if self.selectedPSM == accession:
            self.unfilterPSM()
        else:
            self.filterPSM(accession)

    def PSMClicked(self, item):
        """ sets relevantContent of psm table and decides whether to use filter or not
            ...
            Parameters
            ----------
            item : QTableWidgetItem
                clicked table widget item
        """
        if self.tablePSMFull.isHidden():
            relevantContent = self.PSMFiltered
        else:
            relevantContent = self.PSMFull

        accession = relevantContent[item.row()][2]

        if self.selectedPRT == accession:
            self.unfilterPRT()
        else:
            self.filterPRT(accession)

    def filterPRT(self, accession):
        """ filters prt table
        ...
        Parameters
        ----------
        accession : string
            contains data of accession column of clicked row and therewith information about connected psms 
        """
        self.tablePRTFiltered.setHidden(False)
        self.tablePRTFull.setHidden(True)

        self.selectedPRT = accession

        self.PRTFiltered = [p for p in self.PRTFull if p[0] == self.selectedPRT]
        self.createTable(self.tablePRTFiltered, self.PRTFiltered)

    def filterPSM(self, accession):
        """ filters psm table
        ...
        Parameters
        ----------
        accession : string
            contains data of accession column of clicked row and therewith information about connected prts 
        """
        self.tablePSMFiltered.setHidden(False)
        self.tablePSMFull.setHidden(True)

        self.selectedPSM = accession

        self.PSMFiltered = [p for p in self.PSMFull if p[2] == self.selectedPSM]
        self.createTable(self.tablePSMFiltered, self.PSMFiltered)

    def unfilterPRT(self):
        """ deactivates filter of prt table"""
        self.tablePRTFiltered.setHidden(True)
        self.tablePRTFull.setHidden(False)
        self.selectedPRT = ""
        self.PRTFiltered = []

    def unfilterPSM(self):
        """ deactivates filter of psm table"""
        self.tablePSMFiltered.setHidden(True)
        self.tablePSMFull.setHidden(False)
        self.selectedPSM = ""
        self.PSMFiltered = []

    def browsePRT(self, item):
        """opens uniprot website when prt table item is double clicked """
        if self.tablePRTFull.isHidden():
            accession = self.PRTFiltered[item.row()][0].split("|", 2)[1]
        else:
            accession = self.PRTFull[item.row()][0].split("|", 2)[1]

        webbrowser.open("https://www.uniprot.org/uniprot/" + accession)

    def browsePSM(self, item):
        """opens uniport website when psm table item is double clicked """
        if self.tablePSMFull.isHidden():
            accession = self.PSMFiltered[item.row()][2].split("|", 2)[1]
        else:
            accession = self.PSMFull[item.row()][2].split("|", 2)[1]

        webbrowser.open("https://www.uniprot.org/uniprot/" + accession)



###########################################################################
# Biological example of phosphopeptide-scoring
###########################################################################
from PhosphoScoring import PhosphoScorerSimple, PhosphoScorerAScore
import pyopenms
import csv

pepxml_file = "data/sample_xtandem_output.pep.xml"
mzml_file = "data/sample_spectra.mzML"
output = "sample_ascore_output.csv"
fh = open(output, "w")
writer = csv.writer(fh)

# Cutoff for filtering peptide hits by their score
cutoff_score = 0.75

#
# Data I/O : load the pep.xml file and the mzXML file
#
protein_ids = []
peptide_ids = []
pyopenms.PepXMLFile().load(pepxml_file, protein_ids, peptide_ids)
exp = pyopenms.MSExperiment()
pyopenms.FileHandler().loadExperiment(mzml_file, exp)


def compute_spectrum_bins(exp):
    rt_bins = {}
    for s in exp:
        if s.getMSLevel() == 1:
            continue
        tmp = rt_bins.get(int(s.getRT()), [])
        tmp.append(s)
        rt_bins[int(s.getRT())] = tmp
    return rt_bins


def mapPeptideIdsToSpectra(peptide_ids, exp, matching_mass_tol=1.0):
    rt_bins = compute_spectrum_bins(exp)
    hit_mapping = {}
    for i, pid in enumerate(peptide_ids):
        rt = pid.getMetaValue("RT")
        mz = pid.getMetaValue("MZ")
        # identify the corresponding spectrum from the correct RT bin
        corresponding_spectras = rt_bins[int(rt)]
        for corresponding_spectrum in corresponding_spectras:
            if (
                    abs(
                        corresponding_spectrum.getPrecursors()[0].getMZ() - mz
                        ) <
                    matching_mass_tol
            ):
                hit_mapping[i] = corresponding_spectrum
        if not hit_mapping.hasKey(i):
            print
            "Could not map hit at RT %s and MZ %s" % (rt, mz)
    return hit_mapping


#
# Filter the search results and create the mapping of search results to spectra
#
filtered_ids = [p for p in peptide_ids if p.getHits()[0].getScore() >
                cutoff_score]
# For teaching purposes, only ids betwen 1200 and 1600 s in RT are kept
# (also the spectra are filtered)
filtered_ids = [
    p
    for p in filtered_ids
    if p.getMetaValue("RT") > 1200 and p.getMetaValue("RT") < 1600
]
print
"==========================================================================="
print
"Filtered: kept %s ids below the cutoff score of %s out of %s" % (
    len(filtered_ids),
    cutoff_score,
    len(peptide_ids),
)
hit_mapping = mapPeptideIdsToSpectra(filtered_ids, exp)

#
# Iterate through all peptide hits, extract the corresponding spectra and hand
# it to a scoring function.
#

# Writer CSV header
print
"Will print the original, search-engine sequence," \
    " the AScore sequence and the PhosphoScorerSimple sequence"
writer.writerow(
    [
        "Search-Engine Score",
        "AScore",
        "AScore sequence",
        "Simple Scorer sequence",
        "Old Sequence",
        "# Sites",
    ]
)
for i in range(len(filtered_ids)):
    # Retrieve the input data:
    #  - the peptide hit from the search engine (we take the first hit here)
    #  - the corresponding spectrum
    phit = filtered_ids[i].getHits()[0]
    spectrum = hit_mapping[i]
    nr_sites = phit.getSequence().toString().count("Phospho")

    # Skip non-phospho hits
    if nr_sites == 0:
        continue

    ascore_result = PhosphoScorerAScore().score(phit, spectrum)
    simple_result = PhosphoScorerSimple().score(phit, spectrum)
    print
    "====", phit.getSequence().toString(), ascore_result[
        1
    ].toString(), simple_result[1].toString()

    # Store the resulting hit in our CSV file
    row = [
        phit.getScore(),
        ascore_result[0],
        ascore_result[1].toString(),
        simple_result[1].toString(),
        phit.getSequence().toString(),
        nr_sites,
    ]
    writer.writerow(row)

fh.close()





# Initialize Experiment
from pyopenms.pyopenms_1 import TheoreticalSpectrumGenerator
from pyopenms.pyopenms_2 import MSExperiment
from pyopenms.pyopenms_3 import SpectrumAlignment
from pyopenms.pyopenms_4 import MzMLFile
from pyopenms.pyopenms_5 import AASequence
from pyopenms.pyopenms_7 import MSSpectrum

exp = MSExperiment()

# Load file into experiment container
MzMLFile().load("../data/YIC(Carbamidomethyl)DNQDTISSK.mzML", exp)

# Get first spectrum
spectra = exp.getSpectra()
spectrum = exp[0]

# Generate theoretical spectrum
tsg = TheoreticalSpectrumGenerator()
theo_spectrum = MSSpectrum()
p = tsg.getParameters()
p.setValue(b"add_b_ions", b"true", b"Add peaks of b-ions to the spectrum")
p.setValue(b"add_metainfo", b"true", "")
tsg.setParameters(p)
tsg.getSpectrum(
    theo_spectrum, AASequence.fromString("YIC(Carbamidomethyl)DNQDTISSK"), 1, 2
)

# Compute PSM
alignment = []
spa = SpectrumAlignment()
p = spa.getParameters()
p.setValue(b"tolerance", 0.5)
p.setValue(b"is_relative_tolerance", b"false")  # 0.5 Da tolerance
spa.setParameters(p)
spa.getSpectrumAlignment(alignment, theo_spectrum, spectrum)

# Print matching ions and mz from theoretical spectrum
print("Matched peaks: " + str(len(alignment)))
print("ion\ttheo. m/z\tobserved m/z")

for theo_idx, obs_idx in alignment:
    print(
        theo_spectrum.getStringDataArrays()[0][theo_idx].decode() +
        "\t" +
        str(theo_spectrum[theo_idx].getMZ()) +
        "\t" +
        str(spectrum[obs_idx].getMZ())
    )

# Matched peaks: 16
# ion     theo. m/z       observed m/z
# y2+     234.14483407287105      234.12303161621094
# y5++    268.15794163667096      268.1054382324219
# b2+     277.154670104771        277.24560546875
# y3+     321.17686323237103      321.2969970703125
# y4+     434.26092758327104      434.2879638671875
# b3+     437.185319089971        437.291259765625
# y5+     535.308606806571        535.18896484375
# b4+     552.212263249471        552.3375244140625
# b9++    562.239866948271        562.4213256835938
# y10++   584.2509635120709       584.4120483398438
# y11++   640.7929956875209       640.9539184570312
# b11++   649.2718961077711       649.0973510742188
# y6+     650.335550966071        650.3652954101562
# b5+     666.255191440871        666.1763305664062
# y7+     778.394129221271        778.3391723632812
# b6+     794.3137696960711       794.192138671875
